@@ -52,6 +52,8 @@
 
 #include "gnome-desktop-item.h"
 
+#define sure_string(s) ((s)!=NULL?(s):"")
+
 struct _GnomeDesktopItem {
         int refcount;
 
@@ -866,80 +868,527 @@ replace_percentsign(int argc, char **argv, const char *ps,
 	return ret;
 }
 
-/* the appargv, will get modified but not freed, the replacekdefiles
-   tells us to replace the %f %u and %F for kde files with nothing */
+static char **
+list_to_vector (GSList *list)
+{
+	int len = g_slist_length (list);
+	char **argv;
+	int i;
+	GSList *li;
+
+	argv = g_new0 (char *, len+1);
+
+	for (i = 0, li = list;
+	     li != NULL;
+	     li = li->next, i++) {
+		argv[i] = g_strdup (li->data);
+	}
+	argv[i] = NULL;
+
+	return argv;
+}
+
+static GSList *
+make_args (GList *files)
+{
+	GSList *list = NULL;
+	GList *li;
+
+	for (li = files; li != NULL; li = li->next) {
+		GnomeVFSURI *uri;
+		const char *file = li->data;
+		if (file == NULL)
+			continue;;
+		uri = gnome_vfs_uri_new (file);
+		list = g_slist_prepend (list, uri);
+	}
+
+	return g_slist_reverse (list);
+}
+
+static void
+free_args (GSList *list)
+{
+	GSList *li;
+
+	for (li = list; li != NULL; li = li->next) {
+		GnomeVFSURI *uri = li->data;
+		li->data = NULL;
+		gnome_vfs_uri_unref (uri);
+	}
+	g_slist_free (list);
+}
+
+static char *
+stringify_files (GSList *args)
+{
+	GSList *li;
+	GString *str = g_string_new (NULL);
+	const char *sep = "";
+
+	for (li = args; li != NULL; li = li->next) {
+		GnomeVFSURI *uri = li->data;
+		if (gnome_vfs_uri_is_local (uri)) {
+			const char *path;
+			path = gnome_vfs_uri_get_path (uri);
+			g_string_append (str, sep);
+			g_string_append (str, path);
+			sep = " ";
+		}
+
+	}
+
+	return g_string_free (str, FALSE);
+}
+
+static char *
+stringify_dirs (GSList *args)
+{
+	GSList *li;
+	GString *str = g_string_new (NULL);
+	const char *sep = "";
+
+	for (li = args; li != NULL; li = li->next) {
+		GnomeVFSURI *uri = li->data;
+		if (gnome_vfs_uri_is_local (uri)) {
+			char *path;
+			path = gnome_vfs_uri_extract_dirname (uri);
+			g_string_append (str, sep);
+			g_string_append (str, path);
+			g_free (path);
+			sep = " ";
+		}
+
+	}
+
+	return g_string_free (str, FALSE);
+}
+
+static char *
+stringify_names (GSList *args)
+{
+	GSList *li;
+	GString *str = g_string_new (NULL);
+	const char *sep = "";
+
+	for (li = args; li != NULL; li = li->next) {
+		GnomeVFSURI *uri = li->data;
+		if (gnome_vfs_uri_is_local (uri)) {
+			char *path;
+			path = gnome_vfs_uri_extract_short_path_name (uri);
+			g_string_append (str, sep);
+			g_string_append (str, path);
+			g_free (path);
+			sep = " ";
+		}
+
+	}
+
+	return g_string_free (str, FALSE);
+}
+
+static char *
+stringify_uris (GSList *args)
+{
+	GSList *li;
+	GString *str = g_string_new (NULL);
+
+	for (li = args; li != NULL; li = li->next) {
+		GnomeVFSURI *uri = li->data;
+		char *suri;
+		if (li != args)
+			g_string_append (str, " ");
+
+		suri = gnome_vfs_uri_to_string (uri, 0 /* hide_options */);
+		g_string_append (str, suri);
+		g_free (suri);
+	}
+
+	return g_string_free (str, FALSE);
+}
+
+static GSList *
+append_files (GSList *vector_list,
+	      GSList *args)
+{
+	GSList *li;
+
+	for (li = args; li != NULL; li = li->next) {
+		GnomeVFSURI *uri = li->data;
+		if (gnome_vfs_uri_is_local (uri)) {
+			char *path;
+			path = g_strdup (gnome_vfs_uri_get_path (uri));
+			vector_list = g_slist_append (vector_list, path);
+		}
+
+	}
+
+	return vector_list;
+}
+
+static GSList *
+append_dirs (GSList *vector_list,
+	     GSList *args)
+{
+	GSList *li;
+
+	for (li = args; li != NULL; li = li->next) {
+		GnomeVFSURI *uri = li->data;
+		if (gnome_vfs_uri_is_local (uri)) {
+			char *path;
+			path = gnome_vfs_uri_extract_dirname (uri);
+			if (path == NULL)
+				continue;
+			vector_list = g_slist_append (vector_list, path);
+		}
+	}
+
+	return vector_list;
+}
+
+static GSList *
+append_names (GSList *vector_list,
+	      GSList *args)
+{
+	GSList *li;
+
+	for (li = args; li != NULL; li = li->next) {
+		GnomeVFSURI *uri = li->data;
+		if (gnome_vfs_uri_is_local (uri)) {
+			char *path;
+			path = gnome_vfs_uri_extract_short_path_name (uri);
+			if (path == NULL)
+				continue;
+			vector_list = g_slist_append (vector_list, path);
+		}
+	}
+
+	return vector_list;
+}
+
+static GSList *
+append_uris (GSList *vector_list,
+	     GSList *args)
+{
+	GSList *li;
+
+	for (li = args; li != NULL; li = li->next) {
+		GnomeVFSURI *uri = li->data;
+		char *str;
+
+		str = gnome_vfs_uri_to_string (uri, 0 /* hide_options */);
+		vector_list = g_slist_append (vector_list, str);
+	}
+
+	return vector_list;
+}
+
+static char *
+get_first_file (GSList **arg_ptr)
+{
+	while (*arg_ptr != NULL) {
+		GnomeVFSURI *uri = (*arg_ptr)->data;
+		if (gnome_vfs_uri_is_local (uri)) {
+			char *path;
+			path = g_strdup (gnome_vfs_uri_get_path (uri));
+			return path;
+		}
+		*arg_ptr = (*arg_ptr)->next;
+	}
+
+	return NULL;
+}
+
+static char *
+get_first_dir (GSList **arg_ptr)
+{
+	while (*arg_ptr != NULL) {
+		GnomeVFSURI *uri = (*arg_ptr)->data;
+		if (gnome_vfs_uri_is_local (uri)) {
+			char *path;
+			path = gnome_vfs_uri_extract_dirname (uri);
+			return path;
+		}
+		*arg_ptr = (*arg_ptr)->next;
+	}
+
+	return NULL;
+}
+
+static char *
+get_first_name (GSList **arg_ptr)
+{
+	while (*arg_ptr != NULL) {
+		GnomeVFSURI *uri = (*arg_ptr)->data;
+		if (gnome_vfs_uri_is_local (uri)) {
+			char *path;
+			path = gnome_vfs_uri_extract_short_path_name (uri);
+			return path;
+		}
+		*arg_ptr = (*arg_ptr)->next;
+	}
+
+	return NULL;
+}
+
+static char *
+get_first_uri (GSList **arg_ptr)
+{
+	if (*arg_ptr != NULL) {
+		GnomeVFSURI *uri = (*arg_ptr)->data;
+		return gnome_vfs_uri_to_string (uri, 0 /* hide_options */);
+	}
+
+	return NULL;
+}
+
+enum {
+	ADDED_NONE = 0,
+	ADDED_SINGLE,
+	ADDED_ALL
+};
+
+static char *
+substitute_in_string (const GnomeDesktopItem *item,
+		      const char *arg,
+		      GSList *args,
+		      GSList **arg_ptr,
+		      int *added_status)
+{
+	GString *str = g_string_new (NULL);
+	int i;
+
+	for (i = 0; arg[i] != '\0'; i++) {
+		char *s;
+		const char *cs;
+
+		if (arg[i] != '%' ||
+		    arg[i+1] == '\0') {
+			g_string_append_c (str, arg[i]);
+			continue;
+		}
+
+		i++;
+
+		switch (arg[i]) {
+		/* FIXME: subst also %U, %F, %N, %D */
+		case '%':
+			g_string_append_c (str, '%');
+			break;
+		case 'U':
+			s = stringify_uris (args);
+			g_string_append (str, s);
+			g_free (s);
+			*added_status = ADDED_ALL;
+			break;
+		case 'F':
+			s = stringify_files (args);
+			g_string_append (str, s);
+			g_free (s);
+			*added_status = ADDED_ALL;
+			break;
+		case 'N':
+			s = stringify_names (args);
+			g_string_append (str, s);
+			g_free (s);
+			*added_status = ADDED_ALL;
+			break;
+		case 'D':
+			s = stringify_dirs (args);
+			g_string_append (str, s);
+			g_free (s);
+			*added_status = ADDED_ALL;
+			break;
+		case 'f':
+			s = get_first_file (arg_ptr);
+			g_string_append (str, sure_string (s));
+			g_free (s);
+			if (*added_status != ADDED_ALL)
+				*added_status = ADDED_SINGLE;
+			break;
+		case 'u':
+			s = get_first_uri (arg_ptr);
+			g_string_append (str, sure_string (s));
+			g_free (s);
+			if (*added_status != ADDED_ALL)
+				*added_status = ADDED_SINGLE;
+			break;
+		case 'd':
+			s = get_first_dir (arg_ptr);
+			g_string_append (str, sure_string (s));
+			g_free (s);
+			if (*added_status != ADDED_ALL)
+				*added_status = ADDED_SINGLE;
+			break;
+		case 'n':
+			s = get_first_name (arg_ptr);
+			g_string_append (str, sure_string (s));
+			g_free (s);
+			if (*added_status != ADDED_ALL)
+				*added_status = ADDED_SINGLE;
+			break;
+		case 'm':
+			cs = gnome_desktop_item_get_string (item, GNOME_DESKTOP_ITEM_MINI_ICON);
+			g_string_append (str, sure_string (cs));
+			break;
+		case 'i':
+			s = gnome_desktop_item_get_icon (item);
+			if (s != NULL) {
+				g_string_append (str, sure_string (s));
+				g_free (s);
+			} else {
+				/* else default to just what's in there */
+				cs = gnome_desktop_item_get_string (item, GNOME_DESKTOP_ITEM_ICON);
+				g_string_append (str, sure_string (cs));
+			}
+			break;
+		case 'c':
+			cs = gnome_desktop_item_get_localestring (item, GNOME_DESKTOP_ITEM_COMMENT);
+			g_string_append (str, sure_string (cs));
+			break;
+		case 'k':
+			/* FIXME: is this the name of the .desktop file (location) or the name
+			 * of the entry???? the spec is ambiguous */
+			cs = gnome_desktop_item_get_localestring (item, GNOME_DESKTOP_ITEM_NAME);
+			g_string_append (str, sure_string (cs));
+			break;
+		case 'v':
+			cs = gnome_desktop_item_get_localestring (item, GNOME_DESKTOP_ITEM_DEV);
+			g_string_append (str, sure_string (cs));
+			break;
+		default:
+			break;
+		}
+	}
+
+	return g_string_free (str, FALSE);
+}
+
+static GSList *
+append_app_arg (GSList *vector_list,
+		const GnomeDesktopItem *item,
+		const char *arg,
+		GSList *args,
+		GSList **arg_ptr,
+		int *added_status)
+{
+	if (strchr (arg, '%') == NULL) {
+		return g_slist_append (vector_list, g_strdup (arg));
+	
+	/* First we subst for some where it could be more then
+	 * one argument */
+	} else if (strcmp (arg, "%F")) {
+		*added_status = ADDED_ALL;
+		return append_files (vector_list,
+				     args);
+	} else if (strcmp (arg, "%U")) {
+		*added_status = ADDED_ALL;
+		return append_uris (vector_list,
+				    args);
+	} else if (strcmp (arg, "%D")) {
+		*added_status = ADDED_ALL;
+		return append_dirs (vector_list,
+				    args);
+	} else if (strcmp (arg, "%N")) {
+		*added_status = ADDED_ALL;
+		return append_names (vector_list,
+				     args);
+	} else {
+		char *str = substitute_in_string (item,
+						  arg,
+						  args,
+						  arg_ptr,
+						  added_status);
+		return g_slist_append (vector_list, str);
+	}
+
+}
+
 static int
-ditem_execute(const GnomeDesktopItem *item, int appargc, const char *appargv[], int argc, const char *argv[], gboolean replacekdefiles)
+ditem_execute (const GnomeDesktopItem *item,
+	       int appargc,
+	       char *appargv[],
+	       GList *file_list,
+	       gboolean launch_only_one,
+	       GError **error)
 {
         char **real_argv;
-        int real_argc;
         int i, j, ret;
 	char **term_argv = NULL;
 	int term_argc = 0;
-	GSList *tofree = NULL;
+	GSList *vector_list;
+	GSList *args, *arg_ptr;
+	int added_status;
 
         g_return_val_if_fail (item, -1);
 
-        if(!argv)
-                argc = 0;
+	if (gnome_desktop_item_get_boolean (item, GNOME_DESKTOP_ITEM_TERMINAL)) {
+		const char *options =
+			gnome_desktop_item_get_string (item, GNOME_DESKTOP_ITEM_TERMINAL_OPTIONS);
 
-
-	/* check the type, if there is one set */
-	if(item->type != GNOME_DESKTOP_ITEM_TYPE_APPLICATION) {
-		/* FIXME: ue GError */
-		/* we are not an application so just exit
-		   with an error */
-		g_warning(_("Attempting to launch a non-application"));
-		return -1;
-	}
-
-	if(gnome_desktop_item_get_boolean (item, "Terminal")) {
-		/* FIXME: add temrinal options from desktop file */
-		/* somewhat hackish I guess, we should use this in
-		 * the prepend mode rather then just getting a new
-		 * vector, but I didn't want to rewrite this whole
-		 * function */
 		term_argc = 0;
 		term_argv = NULL;
+
+		if (options != NULL) {
+			g_shell_parse_argv (options,
+					    &term_argc,
+					    &term_argv,
+					    NULL /* error */);
+			/* ignore errors */
+		}
+
 		gnome_prepend_terminal_to_vector (&term_argc, &term_argv);
 	}
 
-        real_argc = term_argc + argc + appargc;
-        real_argv = g_alloca((real_argc + 1) * sizeof(char *));
+	args = make_args (file_list);
+	arg_ptr = make_args (file_list);
 
-        for(i = 0; i < term_argc; i++)
-                real_argv[i] = term_argv[i];
+	do {
+		vector_list = NULL;
+		for(i = 0; i < term_argc; i++)
+			vector_list = g_slist_append (vector_list,
+						      g_strdup (term_argv[i]));
 
-        for(j = 0; j < appargc; j++, i++)
-                real_argv[i] = (char *)appargv[j];
+		added_status = ADDED_NONE;
+		for (i = 0; i < appargc; i++) {
+			vector_list = append_app_arg (vector_list, item,
+						      appargv[i],
+						      args,
+						      &arg_ptr,
+						      &added_status);
+		}
 
-        for(j = 0; j < argc; j++, i++)
-                real_argv[i] = (char *)argv[j];
+		real_argv = list_to_vector (vector_list);
+		g_slist_foreach (vector_list, (GFunc)g_free, NULL);
+		g_slist_free (vector_list);
 
-	/* FIXME: this ought to be standards compliant */
-	/* replace the standard %? thingies */
-	if(replacekdefiles) {
-		replace_percentsign(real_argc, real_argv, "%f", NULL, &tofree, TRUE);
-		replace_percentsign(real_argc, real_argv, "%F", NULL, &tofree, TRUE);
-		replace_percentsign(real_argc, real_argv, "%u", NULL, &tofree, TRUE);
-	}
-	replace_percentsign(real_argc, real_argv, "%d", g_getenv("PWD"), &tofree, FALSE);
-#if 0
-	replace_percentsign(real_argc, real_argv, "%m", 
-			    gnome_desktop_item_get_string(item,"MiniIcon"), &tofree, FALSE);
-#endif
-	replace_percentsign(real_argc, real_argv, "%c", 
-			    gnome_desktop_item_get_localestring(item, "Comment"), &tofree, FALSE);
+		if ( ! g_spawn_async (NULL /* working_directory */,
+				      real_argv,
+				      NULL /* envp */,
+				      G_SPAWN_SEARCH_PATH /* flags */,
+				      NULL /* child_setup */,
+				      NULL /* user_data */,
+				      &ret /* child_pid */,
+				      error)) {
+			/* The error was set for us,
+			 * we just can't launch this thingie */
+			ret = -1;
+			break;
+		}
 
-        ret = gnome_execute_async(NULL, real_argc, real_argv);
+		if (arg_ptr != NULL)
+			arg_ptr = arg_ptr->next;
 
-	if(tofree) {
-		g_slist_foreach(tofree,(GFunc)g_free,NULL);
-		g_slist_free(tofree);
-	}
+	/* rinse, repeat until we run out of arguments (That
+	 * is if we were adding singles anyway) */
+	} while (added_status == ADDED_SINGLE &&
+		 arg_ptr != NULL &&
+		 ! launch_only_one);
 
-	if(term_argv)
-		g_strfreev(term_argv);
+	free_args (args);
+
+	if (term_argv)
+		g_strfreev (term_argv);
 
 	return ret;
 }
@@ -972,339 +1421,143 @@ strip_the_amp (char *exec)
 /**
  * gnome_desktop_item_launch:
  * @item: A desktop item
- * @argc: An optional count of arguments to be added to the arguments defined.
- * @argv: An optional argument array, of length 'argc', to be appended
- *        to the command arguments specified in 'item'. Can be NULL.
+ * @file_list:  Files/URIs to launch this item with, can be %NULL
+ * @flags: FIXME
+ * @error: FIXME
  *
  * This function runs the program listed in the specified 'item',
  * optionally appending additional arguments to its command line.  It uses
- * poptParseArgvString to parse the the exec string into a vector which is
- * then passed to gnome_exec_async for execution with argc and argv appended.
+ * #g_shell_parse_argv to parse the the exec string into a vector which is
+ * then passed to #g_spawn_async for execution. This can return all
+ * the errors from GnomeURL, #g_shell_parse_argv and #g_spawn_async,
+ * in addition to it's own.  The files are
+ * only added if the entry defines one of the standard % strings in it's
+ * Exec field.
  *
- * Returns: The value returned by gnome_execute_async() upon execution of
- * the specified item or -1 on error.  It may also return a 0 on success
- * if pid is not available, such as in a case where the entry is a URL
- * entry.
+ * Returns: The the pid of the process spawned.  If more then one
+ * process was spawned the last pid is returned.  On error -1
+ * is returned and @error is set.
  */
 int
 gnome_desktop_item_launch (const GnomeDesktopItem *item,
-			   int argc,
-			   char **argv,
+			   GList *file_list,
+			   GnomeDesktopItemLaunchFlags flags,
 			   GError **error)
 {
-	const char **temp_argv = NULL;
+	char **temp_argv = NULL;
 	int temp_argc = 0;
 	const char *exec;
 	char *the_exec;
 	int ret;
 
-	exec = gnome_desktop_item_get_string (item, "Exec");
+	exec = gnome_desktop_item_get_string (item, GNOME_DESKTOP_ITEM_EXEC);
 	/* This is a URL, so launch it as a url */
 	if (item->type == GNOME_DESKTOP_ITEM_TYPE_LINK) {
 		const char *url;
-		url = gnome_desktop_item_get_string (item, "URL");
+		url = gnome_desktop_item_get_string (item, GNOME_DESKTOP_ITEM_URL);
 		if (url && url[0] != '\0') {
-			if (gnome_url_show (url, NULL))
+			if (gnome_url_show (url, error))
 				return 0;
 			else
 				return -1;
 		/* Gnome panel used to put this in Exec */
 		} else if (exec && exec[0] != '\0') {
-			if (gnome_url_show (exec, NULL))
+			if (gnome_url_show (exec, error))
 				return 0;
 			else
 				return -1;
 		} else {
+			g_set_error (error,
+				     GNOME_DESKTOP_ITEM_ERROR,
+				     GNOME_DESKTOP_ITEM_ERROR_NO_URL,
+				     _("No URL to launch"));
 			return -1;
 		}
-		/* FIXME: use GError */
 	}
 
-	if(!exec || exec[0] == '\0') {
-		/* FIXME: use GError */
-		g_warning(_("Trying to execute an item with no 'Exec'"));
+	/* check the type, if there is one set */
+	if (item->type != GNOME_DESKTOP_ITEM_TYPE_APPLICATION) {
+		g_set_error (error,
+			     GNOME_DESKTOP_ITEM_ERROR,
+			     GNOME_DESKTOP_ITEM_ERROR_NOT_LAUNCHABLE,
+			     _("Not a launchable item"));
+		return -1;
+	}
+
+
+	if (exec == NULL ||
+	    exec[0] == '\0') {
+		g_set_error (error,
+			     GNOME_DESKTOP_ITEM_ERROR,
+			     GNOME_DESKTOP_ITEM_ERROR_NO_EXEC_STRING,
+			     _("No command (Exec) to launch"));
 		return -1;
 	}
 
 
 	/* make a new copy and get rid of spaces */
-	the_exec = g_alloca(strlen(exec)+1);
-	strcpy(the_exec,exec);
+	the_exec = g_alloca (strlen (exec) + 1);
+	strcpy (the_exec, exec);
 
-	if(!strip_the_amp(the_exec))
-		return -1;
-
-	/* we use a popt function as it does exactly what we want to do and
-	   gnome already uses popt */
-	if(poptParseArgvString(exec, &temp_argc, &temp_argv) != 0) {
-		/* no we have failed in making this a vector what should
-		   we do? we can pretend that the exec failed and return -1,
-		   perhaps we should give a little better error? */
+	if ( ! strip_the_amp (the_exec)) {
+		g_set_error (error,
+			     GNOME_DESKTOP_ITEM_ERROR,
+			     GNOME_DESKTOP_ITEM_ERROR_BAD_EXEC_STRING,
+			     _("Bad command (Exec) to launch"));
 		return -1;
 	}
 
-	ret = ditem_execute (item, temp_argc, temp_argv, argc, argv, TRUE);
+	if ( ! g_shell_parse_argv (exec, &temp_argc, &temp_argv, error)) {
+		/* The error now comes from g_shell_parse_argv */
+		return -1;
+	}
 
-	/* free the array done by poptParseArgvString, not by
-	   g_free but free, this is not a memory leak the whole thing
-	   is one allocation */
-	free(temp_argv);
+	ret = ditem_execute (item, temp_argc, temp_argv, file_list,
+			     (flags & GNOME_DESKTOP_ITEM_LAUNCH_ONLY_ONE),
+			     error);
+
+	g_strfreev (temp_argv);
 
 	return ret;
 }
 
-/* replace %s, %f, %u or %F in exec with file and run */
-static int G_GNUC_UNUSED
-ditem_exec_single_file (GnomeDesktopItem *item, const char *exec, char *file)
-{
-	const char **temp_argv = NULL;
-	int temp_argc = 0;
-	char *the_exec;
-	GSList *tofree = NULL;
-	int ret;
-
-        g_return_val_if_fail (item, -1);
-	if(!exec) {
-		g_warning(_("No exec field for this drop"));
-		return -1;
-	}
-
-	/* make a new copy and get rid of spaces */
-	the_exec = g_alloca(strlen(exec)+1);
-	strcpy(the_exec,exec);
-
-	if(!strip_the_amp(the_exec))
-		return -1;
-
-	/* we use a popt function as it does exactly what we want to do and
-	   gnome already uses popt */
-	if(poptParseArgvString(exec, &temp_argc, &temp_argv) != 0) {
-		/* no we have failed in making this a vector what should
-		   we do? we can pretend that the exec failed and return -1,
-		   perhaps we should give a little better error? */
-		return -1;
-	}
-
-	/* we are gnomish and we want to use %s, but if someone uses %f, %u or %F
-	   we forgive them and do it too */
-	if(replace_percentsign(temp_argc, (char **)temp_argv, "%s", file, &tofree,TRUE))
-		goto perc_replaced;
-	if(replace_percentsign(temp_argc, (char **)temp_argv, "%f", file, &tofree,TRUE))
-		goto perc_replaced;
-	if(replace_percentsign(temp_argc, (char **)temp_argv, "%u", file, &tofree,TRUE))
-		goto perc_replaced;
-	if(replace_percentsign(temp_argc, (char **)temp_argv, "%F", file, &tofree,TRUE))
-		goto perc_replaced;
-	g_free(temp_argv);
-	g_warning(_("Nothing to replace with a file in the exec string"));
-	return -1;
-perc_replaced:
-
-	ret = ditem_execute(item, temp_argc, temp_argv, 0, NULL, FALSE);
-
-	if(tofree) {
-		g_slist_foreach(tofree,(GFunc)g_free,NULL);
-		g_slist_free(tofree);
-	}
-
-	/* free the array done by poptParseArgvString, not by
-	   g_free but free, this is not a memory leak the whole thing
-	   is one allocation */
-	free(temp_argv);
-
-	return ret;
-}
-
-/* works sort of like strcmp(g_strstrip(s1),s2), but returns only
- * TRUE or FALSE and does not modify s1, basically it tells if s2
- * is the same as s1 stripped */
-static gboolean
-stripstreq(const char *s1, const char *s2)
-{
-	size_t len2;
-
-	/* skip over initial spaces */
-	while(*s1 == ' ' || *s1 == '\t')
-		s1++;
-	len2 = strlen(s2);
-	/* compare inner parts */
-	if(strncmp(s1, s2, len2) != 0)
-		return FALSE;
-	/* skip over beyond the equal parts */
-	s1 += len2;
-	for(; *s1; s1++) {
-		/* if we find anything but whitespace that bad */
-		if(*s1 != ' ' && *s1 != '\t')
-			return FALSE;
-	}
-
-	return TRUE;
-}
-
-/* replace %s, %f, %u or %F in exec with files and run, the trick here is
-   that we will only replace %? in case it is a complete argument */
-static int G_GNUC_UNUSED
-ditem_exec_multiple_files (GnomeDesktopItem *item, const char *exec,
-			   GList *files)
-{
-	const char **temp_argv = NULL;
-	int temp_argc = 0;
-	char *the_exec;
-	int ret;
-	int real_argc;
-	const char **real_argv;
-	int i,j;
-	gboolean replaced = FALSE;
-
-        g_return_val_if_fail (item, -1);
-	if(!exec) {
-		g_warning(_("No exec field for this drop"));
-		return -1;
-	}
-
-	/* make a new copy and get rid of spaces */
-	the_exec = g_alloca(strlen(exec)+1);
-	strcpy(the_exec,exec);
-
-	if(!strip_the_amp(the_exec))
-		return -1;
-
-	/* we use a popt function as it does exactly what we want to do and
-	   gnome already uses popt */
-	if(poptParseArgvString(exec, &temp_argc, &temp_argv) != 0) {
-		/* no we have failed in making this a vector what should
-		   we do? we can pretend that the exec failed and return -1,
-		   perhaps we should give a little better error? */
-		return -1;
-	}
-
-	real_argc = temp_argc - 1 + g_list_length(files);
-	/* we allocate +2, one for NULL, one as a buffer in case
-	   we don't find anything to replace */
-	real_argv = g_alloca((real_argc + 2) * sizeof(char *));
-
-	for(i=0,j=0; i < real_argc && j < temp_argc; i++, j++) {
-		GList *li;
-		real_argv[i] = temp_argv[j];
-		/* we only replace once */
-		if(replaced)
-			continue;
-
-		if(!stripstreq(real_argv[i],"%s") &&
-		   !stripstreq(real_argv[i],"%f")!=0 &&
-		   !stripstreq(real_argv[i],"%u")!=0 &&
-		   !stripstreq(real_argv[i],"%F")!=0)
-			continue;
-
-		/* replace the argument with the files we got passed */
-		for(li = files; li!=NULL; li=li->next)
-			real_argv[i++] = li->data;
-
-		/* since we will increment in the for loop itself */
-		i--;
-
-		/* don't replace again! */
-		replaced = TRUE;
-	}
-
-	/* we haven't replaced anything */
-	if(!replaced) {
-		g_warning(_("There was no argument to replace with dropped files"));
-		return -1;
-	}
-
-	ret = ditem_execute(item, real_argc, real_argv, 0, NULL, FALSE);
-
-	/* free the array done by poptParseArgvString, not by
-	   g_free but free, this is not a memory leak the whole thing
-	   is one allocation */
-	free(temp_argv);
-
-	return ret;
-}
-
-
 /**
  * gnome_desktop_item_drop_uri_list:
  * @item: A desktop item
- * @uri_list: an uri_list as if gotten from #gnome_uri_list_extract_uris
+ * @uri_list: text as gotten from a text/uri-list
+ * @flags: FIXME
+ * @error: FIXME
  *
  * A list of files or urls dropped onto an icon, the proper (Url or File)
- * exec is run you can pass directly the output of 
- * #gnome_uri_list_extract_uris.  The input thus must be a GList of
- * strings in the URI format.  If the ditem didn't specify any DND
- * options we assume old style and drop by appending the list to the end
- * of 'Exec' (see #gnome_desktop_item_get_command).  If there were any urls
- * dropped the item must have specified being able to run URLs or you'll get
- * a warning and an error.  Basically if there are ANY urls in the list
- * it's taken as a list of urls and works only on apps that can take urls.
- * If the program can't take that many arguments, as many instances as
- * necessary will be started.
+ * exec is run you can pass directly string that you got as the
+ * text/uri-list.  This just parses the list and calls 
  *
- * Returns: The value returned by gnome_execute_async() upon execution of
+ * Returns: The value returned by #gnome_execute_async() upon execution of
  * the specified item or -1 on error.  If multiple instances are run, the
  * return of the last one is returned.
  */
-/*FIXME: perhaps we should be able to handle mixed url/file lists better
-  rather then treating it as a list of urls */
 int
 gnome_desktop_item_drop_uri_list (const GnomeDesktopItem *item,
-				  GList *uri_list,
+				  const char *uri_list,
+				  GnomeDesktopItemLaunchFlags flags,
 				  GError **error)
 {
-	/* FIXME: */
-#if 0
 	GList *li;
-	gboolean G_GNUC_UNUSED any_urls = FALSE;
-	GList *file_list = NULL;
-	int ret = -1;
+	int ret;
+	GList *list = gnome_vfs_uri_list_parse (uri_list);
 
-	g_return_val_if_fail (item != NULL, -1);
-	g_return_val_if_fail (uri_list != NULL, -1);
-
-	/* How could you drop something on a URL entry, that would
-	 * be bollocks */
-	if (item->entry->Type == GNOME_Desktop_ENTRY_TYPE_URL) {
-		/*FIXME: use a GError */
-		return -1;
+	for (li = list; li != NULL; li = li->next) {
+		GnomeVFSURI *uri = li->data;
+		li->data = gnome_vfs_uri_to_string (uri, 0 /* hide_options */);
+		gnome_vfs_uri_unref (uri);
 	}
 
-	/* FIXME: we should really test if this was a compliant entry and there
-	 * were no %f %u or %F or whatnot, we shouldn't allow a drop!,
-	 * in that case I suppose, we should do that during load and add a
-	 * %F or something */
+	ret = gnome_desktop_item_launch (item, list, flags, error);
 
-#if 0
-	/* the simple case */
-	if(item->item_flags & GNOME_DESKTOP_ITEM_OLD_STYLE_DROP) {
-		int i;
-		int argc = g_list_length(uri_list);
-		const char **argv = g_alloca((argc + 1) * sizeof(char *));
-
-		for(i=0,li=uri_list; li; i++, li=li->next) {
-			argv[i] = li->data;
-		}
-		argv[i] = NULL;
-		return gnome_desktop_item_launch(item, argc, argv);
-	}
-#endif
-
-	/* figure out if we have any urls in the dropped files,
-	   this also builds a list of all the files */
-	for(li=uri_list; li; li=li->next) {
-		file_list = g_list_prepend(file_list, g_strdup (li->data));
-	}
-	if(file_list)
-		file_list = g_list_reverse(file_list);
-
-	if(file_list) {
-		g_list_foreach(file_list,(GFunc)g_free,NULL);
-		g_list_free(file_list);
-	}
+	g_list_foreach (list, (GFunc)g_free, NULL);
+	g_list_free (list);
 
 	return ret;
-#endif
-	return -1;
 }
 
 /**
@@ -1483,6 +1736,16 @@ gnome_desktop_item_get_file_status (const GnomeDesktopItem *item)
 }
 
 static char *kde_icondir = NULL;
+static GSList *hicolor_kde_48 = NULL;
+static GSList *hicolor_kde_32 = NULL;
+static GSList *hicolor_kde_22 = NULL;
+static GSList *hicolor_kde_16 = NULL;
+/* XXX: maybe we don't care about locolor
+static GSList *locolor_kde_48 = NULL;
+static GSList *locolor_kde_32 = NULL;
+static GSList *locolor_kde_22 = NULL;
+static GSList *locolor_kde_16 = NULL;
+*/
 
 static GSList *
 add_dirs (GSList *list, const char *dirname)
@@ -1494,46 +1757,101 @@ add_dirs (GSList *list, const char *dirname)
 	if (dir == NULL)
 		return list;
 
+	list = g_slist_prepend (list, g_strdup (dirname));
+
 	while ((dent = readdir (dir)) != NULL) {
-		char *full = g_build_filename (dirname, dent->d_name, NULL);
-		if (g_file_test (full, G_FILE_TEST_IS_DIR))
+		char *full;
+
+		/* skip hidden and self/parent references */
+		if (dent->d_name[0] == '.')
+			continue;
+
+		full = g_build_filename (dirname, dent->d_name, NULL);
+		if (g_file_test (full, G_FILE_TEST_IS_DIR)) {
 			list = g_slist_prepend (list, full);
-		else
+			list = add_dirs (list, full);
+		} else {
 			g_free (full);
+		}
 	}
 	closedir (dir);
 
 	return list;
 }
 
+static void
+init_kde_dirs (void)
+{
+	char *dirname;
+
+	if (kde_icondir == NULL)
+		return;
+
+#define ADD_DIRS(color,size) \
+	dirname = g_build_filename (kde_icondir, #color,	\
+				    #size "x" #size , NULL);	\
+	color ## _kde_ ## size = add_dirs (NULL, dirname);	\
+	g_free (dirname);
+
+	ADD_DIRS (hicolor, 48);
+	ADD_DIRS (hicolor, 32);
+	ADD_DIRS (hicolor, 22);
+	ADD_DIRS (hicolor, 16);
+
+/* XXX: maybe we don't care about locolor
+	ADD_DIRS (locolor, 48);
+	ADD_DIRS (locolor, 32);
+	ADD_DIRS (locolor, 22);
+	*/
+
+#undef ADD_DIRS
+}
+
 static GSList *
-get_kde_dirs (void)
+get_kde_dirs (int size)
 {
 	GSList *list = NULL;
-	char *dirname;
 
 	if (kde_icondir == NULL)
 		return NULL;
 
-	list = g_slist_prepend (list, kde_icondir);
+	if (size > 32) {
+		/* 48-inf */
+		list = g_slist_concat (g_slist_copy (hicolor_kde_48),
+				       g_slist_copy (hicolor_kde_32));
+		list = g_slist_concat (list,
+				       g_slist_copy (hicolor_kde_22));
+		list = g_slist_concat (list,
+				       g_slist_copy (hicolor_kde_16));
+	} else if (size > 22) {
+		/* 23-32 */
+		list = g_slist_concat (g_slist_copy (hicolor_kde_32),
+				       g_slist_copy (hicolor_kde_48));
+		list = g_slist_concat (list,
+				       g_slist_copy (hicolor_kde_22));
+		list = g_slist_concat (list,
+				       g_slist_copy (hicolor_kde_16));
+	} else if (size > 16) {
+		/* 17-22 */
+		list = g_slist_concat (g_slist_copy (hicolor_kde_22),
+				       g_slist_copy (hicolor_kde_32));
+		list = g_slist_concat (list,
+				       g_slist_copy (hicolor_kde_48));
+		list = g_slist_concat (list,
+				       g_slist_copy (hicolor_kde_16));
+	} else {
+		/* 1-16 */
+		list = g_slist_concat (g_slist_copy (hicolor_kde_16),
+				       g_slist_copy (hicolor_kde_22));
+		list = g_slist_concat (list,
+				       g_slist_copy (hicolor_kde_32));
+		list = g_slist_concat (list,
+				       g_slist_copy (hicolor_kde_48));
+	}
 
-	dirname = g_build_filename (kde_icondir, "hicolor", "48x48", NULL);
-	list = add_dirs (list, dirname);
-	g_free (dirname);
+	list = g_slist_append (list, kde_icondir);
 
-	dirname = g_build_filename (kde_icondir, "hicolor", "32x32", NULL);
-	list = add_dirs (list, dirname);
-	g_free (dirname);
-
-	dirname = g_build_filename (kde_icondir, "locolor", "48x48", NULL);
-	list = add_dirs (list, dirname);
-	g_free (dirname);
-
-	dirname = g_build_filename (kde_icondir, "locolor", "32x32", NULL);
-	list = add_dirs (list, dirname);
-	g_free (dirname);
-
-	return g_slist_reverse (list);
+	return list;
 }
 
 /* similar function is in panel/main.c */
@@ -1541,7 +1859,7 @@ static void
 find_kde_directory (void)
 {
 	int i;
-	const char *kdedir = g_getenv ("KDEDIR");
+	const char *kdedir;
 	char *try_prefixes[] = {
 		"/usr",
 		"/opt/kde",
@@ -1551,14 +1869,22 @@ find_kde_directory (void)
 		"/kde2",
 		NULL
 	};
+
+	if (kde_icondir != NULL)
+		return;
+
+	kdedir = g_getenv ("KDEDIR");
+
 	if (kdedir != NULL) {
 		kde_icondir = g_build_filename (kdedir, "share", "icons", NULL);
+		init_kde_dirs ();
 		return;
 	}
 
 	/* if what configure gave us works use that */
-	if (g_file_test (KDE_MENUDIR, G_FILE_TEST_IS_DIR)) {
+	if (g_file_test (KDE_ICONDIR, G_FILE_TEST_IS_DIR)) {
 		kde_icondir = g_strdup (KDE_ICONDIR);
+		init_kde_dirs ();
 		return;
 	}
 
@@ -1568,6 +1894,7 @@ find_kde_directory (void)
 		if (g_file_test (try, G_FILE_TEST_IS_DIR)) {
 			g_free (try);
 			kde_icondir = g_build_filename (try_prefixes[i], "share", "icons", NULL);
+			init_kde_dirs ();
 			return;
 		}
 		g_free (try);
@@ -1576,6 +1903,87 @@ find_kde_directory (void)
 	/* absolute fallback, these don't exist, but we're out of options
 	   here */
 	kde_icondir = g_strdup (KDE_ICONDIR);
+
+	init_kde_dirs ();
+}
+
+/**
+ * gnome_desktop_item_find_icon:
+ * @icon: icon name, something you'd get out of the Icon key
+ * @desired_size: FIXME
+ * @flags: FIXME
+ *
+ * Description:  This function goes and looks for the icon file.  If the icon
+ * is not an absolute filename, this will look for it in the standard places.
+ * If it can't find the icon, it will return %NULL
+ *
+ * Returns: A newly allocated string
+ */
+char *
+gnome_desktop_item_find_icon (const char *icon,
+			      int desired_size,
+			      int flags)
+{
+	if (icon == NULL) {
+		return NULL;
+	} else if (g_path_is_absolute (icon)) {
+		if (g_file_test (icon, G_FILE_TEST_EXISTS)) {
+			return g_strdup (icon);
+		} else {
+			return NULL;
+		}
+	} else {
+		char *full;
+		GSList *kde_dirs = NULL;
+		GSList *li;
+		const char *exts[] = { ".png", ".xpm", NULL };
+		const char *no_exts[] = { "", NULL };
+		const char **check_exts;
+
+		full = gnome_program_locate_file (NULL,
+						  GNOME_FILE_DOMAIN_PIXMAP,
+						  icon,
+						  TRUE /* only_if_exists */,
+						  NULL /* ret_locations */);
+		if (full == NULL)
+			full = gnome_program_locate_file (NULL,
+							  GNOME_FILE_DOMAIN_APP_PIXMAP,
+							  icon,
+							  TRUE /* only_if_exists */,
+							  NULL /* ret_locations */);
+
+		/* if we found something or no kde, then just return now */
+		if (full != NULL ||
+		    flags & GNOME_DESKTOP_ITEM_ICON_NO_KDE)
+			return full;
+
+		/* If there is an extention don't add any extensions */
+		if (strchr (icon, '.') != NULL) {
+			check_exts = no_exts;
+		} else {
+			check_exts = exts;
+		}
+
+		find_kde_directory ();
+
+		kde_dirs = get_kde_dirs (desired_size);
+
+		for (li = kde_dirs; full == NULL && li != NULL; li = li->next) {
+			int i;
+			for (i = 0; full == NULL && check_exts[i] != NULL; i++) {
+				full = g_strconcat (li->data, G_DIR_SEPARATOR_S, icon,
+						    check_exts[i], NULL);
+				if ( ! g_file_test (full, G_FILE_TEST_EXISTS)) {
+					g_free (full);
+					full = NULL;
+				}
+			}
+		}
+
+		g_slist_free (kde_dirs);
+
+		return full;
+	}
 }
 
 /**
@@ -1591,66 +1999,18 @@ find_kde_directory (void)
 char *
 gnome_desktop_item_get_icon (const GnomeDesktopItem *item)
 {
-/* Similar function is in quick-desktop-reader */
+	/* maybe this function should be deprecated in favour of find icon
+	 * -George */
 	const char *icon;
-	static gboolean checked_kde = FALSE;
 
         g_return_val_if_fail (item != NULL, NULL);
         g_return_val_if_fail (item->refcount > 0, NULL);
 
 	icon = gnome_desktop_item_get_string (item, GNOME_DESKTOP_ITEM_ICON);
 
-	if (icon == NULL) {
-		return NULL;
-	} else if (g_path_is_absolute (icon)) {
-		if (g_file_test (icon, G_FILE_TEST_EXISTS)) {
-			return g_strdup (icon);
-		} else {
-			return NULL;
-		}
-	} else {
-		char *full;
-		char *name = NULL;
-		static GSList *kde_dirs = NULL;
-		GSList *li;
-
-		/* FIXME: no extention, use .png, this is wrong,
-		 * we need to be smarter here */
-		if (strchr (icon, '.') == NULL) {
-			name = g_strconcat (icon, ".png", NULL);
-			icon = name;
-		}
-	       
-		full = gnome_program_locate_file (NULL,
-						  GNOME_FILE_DOMAIN_PIXMAP,
-						  icon,
-						  TRUE /* only_if_exists */,
-						  NULL /* ret_locations */);
-		if (full == NULL)
-			full = gnome_program_locate_file (NULL,
-							  GNOME_FILE_DOMAIN_APP_PIXMAP,
-							  icon,
-							  TRUE /* only_if_exists */,
-							  NULL /* ret_locations */);
-
-		if ( ! checked_kde) {
-			find_kde_directory ();
-			checked_kde = TRUE;
-			kde_dirs = get_kde_dirs ();
-		}
-
-		for (li = kde_dirs; full == NULL && li != NULL; li = li->next) {
-			full = g_build_filename (li->data, icon, NULL);
-			if ( ! g_file_test (full, G_FILE_TEST_EXISTS)) {
-				g_free (full);
-				full = NULL;
-			}
-		}
-
-		g_free (name);
-
-		return full;
-	}
+	return gnome_desktop_item_find_icon (icon,
+					     48 /* desired_size */,
+					     0 /* flags */);
 }
 
 /**
@@ -1717,6 +2077,31 @@ gnome_desktop_item_set_location (GnomeDesktopItem *item, const char *location)
 	item->modified = TRUE;
 }
 
+/**
+ * gnome_desktop_item_set_location_file:
+ * @item: A desktop item
+ * @file: A local filename specifying the file location of this particular item.
+ *
+ * Set's the 'location' uri of this item to the given @file.
+ *
+ * Returns:
+ */
+void
+gnome_desktop_item_set_location_file (GnomeDesktopItem *item, const char *file)
+{
+        g_return_if_fail (item != NULL);
+        g_return_if_fail (item->refcount > 0);
+
+	if (file != NULL) {
+		char *uri;
+		uri = gnome_vfs_get_uri_from_local_path (file);
+		gnome_desktop_item_set_location_file (item, uri);
+		g_free (uri);
+	} else {
+		gnome_desktop_item_set_location_file (item, NULL);
+	}
+}
+
 /*
  * Reading/Writing different sections, NULL is the standard section
  */
