@@ -8,7 +8,7 @@
 #include "config.h"
 #include "panel-marshal.h"
 #include "aligned-widget.h"
-#include "panel_config_global.h"
+#include "panel-config-global.h"
 #include "foobar-widget.h"
 #include "multiscreen-stuff.h"
 #include "panel-typebuiltins.h"
@@ -290,14 +290,12 @@ aligned_widget_change_params (AlignedWidget *aligned,
 			      int sz,
 			      BasePMode mode,
 			      BasePState state,
-			      BasePLevel level,
-			      gboolean avoid_on_maximize,
 			      gboolean hidebuttons_enabled,
 			      gboolean hidebutton_pixmaps_enabled,
 			      PanelBackType back_type,
 			      char *pixmap_name,
 			      gboolean fit_pixmap_bg,
-			      gboolean strech_pixmap_bg,
+			      gboolean stretch_pixmap_bg,
 			      gboolean rotate_pixmap_bg,
 			      GdkColor *back_color)
 {
@@ -316,14 +314,12 @@ aligned_widget_change_params (AlignedWidget *aligned,
 				     sz,
 				     mode,
 				     state,
-				     level,
-				     avoid_on_maximize,
 				     hidebuttons_enabled,
 				     hidebutton_pixmaps_enabled,
 				     back_type,
 				     pixmap_name,
 				     fit_pixmap_bg,
-				     strech_pixmap_bg,
+				     stretch_pixmap_bg,
 				     rotate_pixmap_bg,
 				     back_color);
 }
@@ -346,14 +342,12 @@ aligned_widget_change_align (AlignedWidget *aligned,
 				      BORDER_POS (pos)->edge,
 				      panel->sz, basep->mode,
 				      basep->state,
-				      basep->level,
-				      basep->avoid_on_maximize,
 				      basep->hidebuttons_enabled,
 				      basep->hidebutton_pixmaps_enabled,
 				      panel->back_type,
 				      panel->back_pixmap,
 				      panel->fit_pixmap_bg,
-				      panel->strech_pixmap_bg,
+				      panel->stretch_pixmap_bg,
 				      panel->rotate_pixmap_bg,
 				      &panel->back_color);
 }
@@ -373,43 +367,41 @@ aligned_widget_change_align_edge (AlignedWidget *aligned,
 				      panel->sz,
 				      basep->mode,
 				      basep->state,
-				      basep->level,
-				      basep->avoid_on_maximize,
 				      basep->hidebuttons_enabled,
 				      basep->hidebutton_pixmaps_enabled,
 				      panel->back_type,
 				      panel->back_pixmap,
 				      panel->fit_pixmap_bg,
-				      panel->strech_pixmap_bg,
+				      panel->stretch_pixmap_bg,
 				      panel->rotate_pixmap_bg,
 				      &panel->back_color);
 }
 				 
 GtkWidget *
-aligned_widget_new (int screen,
+aligned_widget_new (gchar *panel_id,
+		    int screen,
 		    AlignedAlignment align,
 		    BorderEdge edge,
 		    BasePMode mode,
 		    BasePState state,
-		    BasePLevel level,
-		    gboolean avoid_on_maximize,
 		    int sz,
 		    gboolean hidebuttons_enabled,
 		    gboolean hidebutton_pixmaps_enabled,
 		    PanelBackType back_type,
 		    char *back_pixmap,
 		    gboolean fit_pixmap_bg,
-		    gboolean strech_pixmap_bg,
+		    gboolean stretch_pixmap_bg,
 		    gboolean rotate_pixmap_bg,
 		    GdkColor *back_color)
 {
-	AlignedWidget *aligned = gtk_type_new (ALIGNED_TYPE_WIDGET);
-	AlignedPos *pos = gtk_type_new (ALIGNED_TYPE_POS);
+	AlignedWidget *aligned = g_object_new (ALIGNED_TYPE_WIDGET, NULL);
+	AlignedPos *pos = g_object_new (ALIGNED_TYPE_POS, NULL);
 	pos->align = align;
 
 	BASEP_WIDGET (aligned)->pos = BASEP_POS (pos);
 
-	border_widget_construct (BORDER_WIDGET (aligned),
+	border_widget_construct (panel_id,
+				 BORDER_WIDGET (aligned),
 				 screen,
 				 edge, 
 				 TRUE,
@@ -417,14 +409,12 @@ aligned_widget_new (int screen,
 				 sz,
 				 mode,
 				 state,
-				 level,
-				 avoid_on_maximize,
 				 hidebuttons_enabled,
 				 hidebutton_pixmaps_enabled,
 				 back_type,
 				 back_pixmap,
 				 fit_pixmap_bg,
-				 strech_pixmap_bg,
+				 stretch_pixmap_bg,
 				 rotate_pixmap_bg,
 				 back_color);
 
