@@ -17,10 +17,12 @@
 #include "button-widget.h"
 #include "drawer.h"
 #include "launcher.h"
+#include "logout.h"
 #include "menu-properties.h"
 #include "menu-util.h"
-#include "panel_config.h"
-#include "panel_config_global.h"
+#include "panel-config.h"
+#include "panel-gconf.h"
+#include "panel-config-global.h"
 #include "session.h"
 #include "status.h"
 #include "swallow.h"
@@ -40,6 +42,18 @@ extern int need_complete_save;
 extern gboolean commie_mode;
 extern GlobalConfig global_config;
 
+static GConfEnumStringPair object_type_enum_map [] = {
+	{ APPLET_DRAWER,   "drawer-object" },
+	{ APPLET_MENU,     "menu-object" },
+	{ APPLET_LOGOUT,   "logout-object" },
+	{ APPLET_SWALLOW,  "swallow-object" },
+	{ APPLET_LAUNCHER, "launcher-object" }, 
+	{ APPLET_EMPTY,    "empty-object" },
+	{ APPLET_LOCK,     "lock-object" },
+	{ APPLET_STATUS,   "status-object" },
+	{ APPLET_BONOBO,   "bonobo-applet" },
+};
+
 static void
 move_applet_callback (GtkWidget *widget, AppletInfo *info)
 {
@@ -56,12 +70,67 @@ move_applet_callback (GtkWidget *widget, AppletInfo *info)
 					PW_DRAG_OFF_CENTER);
 }
 
+static void
+panel_applet_clean_gconf (AppletInfo *info)
+{
+	GConfClient *client;
+	GSList      *id_list, *l;
+	char        *temp_key = NULL;
+	char        *profile;
+
+	g_return_if_fail (info && info->gconf_key);
+
+        client  = panel_gconf_get_client ();
+	profile = session_get_current_profile ();
+
+	if (info->type == APPLET_BONOBO)
+		temp_key = panel_gconf_general_profile_get_full_key (profile, "applet-id-list");
+	else
+		temp_key = panel_gconf_general_profile_get_full_key (profile, "object-id-list");
+
+        id_list = gconf_client_get_list (panel_gconf_get_client (), temp_key, GCONF_VALUE_STRING, NULL);
+
+        for (l = id_list; l; l = l->next)
+                if (!strcmp (info->gconf_key, (char *) l->data))
+                        break;
+
+        if (l) {
+		char *tmp;
+
+                id_list = g_slist_remove_link (id_list, l);
+
+		g_free (l->data);
+		g_slist_free (l);
+
+                gconf_client_set_list (client, temp_key, GCONF_VALUE_STRING, id_list, NULL);
+
+		if (info->type == APPLET_BONOBO)
+			tmp = g_strdup_printf ("/apps/panel/profiles/%s/applets/%s",
+					       profile, info->gconf_key);
+		else
+			tmp = g_strdup_printf ("/apps/panel/profiles/%s/objects/%s",
+					       profile, info->gconf_key);
+
+		panel_gconf_directory_recursive_clean (client, tmp);
+
+		g_free (tmp);
+        }
+
+        g_free (temp_key);
+        g_slist_foreach (id_list, (GFunc) g_free, NULL);
+        g_slist_free (id_list);
+}
+
 /*destroy widgets and call the above cleanup function*/
 void
 panel_applet_clean (AppletInfo *info)
 {
 	g_return_if_fail (info != NULL);
 
+	panel_applet_clean_gconf (info);
+
+	applets = g_slist_remove (applets, info);
+
 	if (info->widget != NULL) {
 		if(info->type == APPLET_STATUS) {
 			status_applet_put_offscreen (info->data);
@@ -72,6 +141,10 @@ panel_applet_clean (AppletInfo *info)
 	}
 
 	info->data = NULL;
+
+	/*
+	 * FIXME: where are we supposed to free the structure?
+	 */
 }
 
 static gboolean
@@ -79,18 +152,12 @@ applet_idle_remove (gpointer data)
 {
 	AppletInfo *info = data;
 
-	switch (info->type) {
-	case APPLET_BONOBO:
-		panel_applet_frame_save_position (PANEL_APPLET_FRAME (info->data));
-		break;
-	case APPLET_SWALLOW: {
+	panel_applet_save_position (info, info->gconf_key);
+
+	if (info->type == APPLET_SWALLOW) {
 		Swallow *swallow = info->data;
 
 		swallow->clean_remove = TRUE;
-		}
-		break;
-	default:
-		break;
 	}
 
 	panel_applet_clean (info);
@@ -158,10 +225,11 @@ applet_callback_callback(GtkWidget *widget, gpointer data)
 		if(strcmp(menu->name,"properties")==0)
 			menu_properties(menu->info->data);
 		else if(strcmp(menu->name,"edit_menus")==0) {
-			char *tmp;
-			if((tmp = g_find_program_in_path ("gmenu")))  {
-				gnome_execute_async (g_get_home_dir (), 1, &tmp);
-				g_free(tmp);
+			char *tmp[3] = { NULL, NULL, NULL };
+			if((tmp[0] = g_find_program_in_path ("nautilus")) != NULL)  {
+				tmp[1] = "applications:/";
+				gnome_execute_async (g_get_home_dir (), 2, tmp);
+				g_free (tmp[0]);
 			}
 		} else if (strcmp (menu->name, "help") == 0) {
 			Menu *menu2 = menu->info->data;
@@ -209,10 +277,6 @@ applet_callback_callback(GtkWidget *widget, gpointer data)
 		if (strcmp (menu->name, "help") == 0)
 			panel_show_help ("specialobjects", "STATUSDOC");
 		break;
-	case APPLET_RUN:
-		if (strcmp (menu->name, "help") == 0)
-			panel_show_help ("specialobjects", "RUNBUTTON");
-		break;
 	case APPLET_BONOBO:
 		/*
 		 * Applet's menu's are handled differently
@@ -259,7 +323,7 @@ panel_applet_add_callback (AppletInfo *info,
 	AppletUserMenu *menu;
 
 	g_return_if_fail (info != NULL);
-	
+
 	menu = panel_applet_get_callback (info->user_menu, callback_name);
 	if (menu == NULL) {
 		menu = g_new0 (AppletUserMenu, 1);
@@ -329,41 +393,13 @@ panel_applet_remove_callback (AppletInfo *info,
 	}
 }
 
-void
-panel_applet_callback_set_sensitive (AppletInfo  *info,
-				     const gchar *callback_name,
-				     gint         sensitive)
-{
-	AppletUserMenu *menu;
-
-	g_return_if_fail(info != NULL);
-	
-	menu = panel_applet_get_callback(info->user_menu, callback_name);
-	if (menu != NULL)
-		menu->sensitive = sensitive;
-	else
-		return; /*it just isn't there*/
-
-	/*make sure the menu is rebuilt*/
-	if(info->menu) {
-		GList *list;
-		for(list=info->user_menu;list!=NULL;list=g_list_next(list)) {
-			AppletUserMenu *menu = list->data;
-			menu->menuitem=NULL;
-			menu->submenu=NULL;
-		}
-		gtk_widget_unref(info->menu);
-		info->menu = NULL;
-		info->menu_age = 0;
-	}
-}
-
 static void
 setup_an_item(AppletUserMenu *menu,
 	      GtkWidget *submenu,
 	      int is_submenu)
 {
-	menu->menuitem = gtk_menu_item_new ();
+	menu->menuitem = gtk_image_menu_item_new ();
+
 	g_signal_connect (G_OBJECT (menu->menuitem), "destroy",
 			  G_CALLBACK (gtk_widget_destroyed),
 			  &menu->menuitem);
@@ -477,13 +513,31 @@ applet_setup_panel_menu (gboolean is_basep)
 	GtkWidget *panel_menu;
 	gchar     *pixmap_path;
 
-	menuitem = gtk_menu_item_new ();
+	menuitem = gtk_image_menu_item_new ();
 
 	pixmap_path = gnome_program_locate_file (NULL, GNOME_FILE_DOMAIN_PIXMAP, 
 						 "gnome-panel.png", TRUE, NULL);
 	if (pixmap_path) {
+		GdkPixbuf *pixbuf, *scaled_pixbuf = NULL;
+		GtkWidget *image = NULL;
+		
+		pixbuf = gdk_pixbuf_new_from_file (pixmap_path, NULL);
+
+		if (pixbuf != NULL) {
+			scaled_pixbuf = gdk_pixbuf_scale_simple (pixbuf,
+								 SMALL_ICON_SIZE,
+								 SMALL_ICON_SIZE,
+								 GDK_INTERP_BILINEAR);
+			g_object_unref (pixbuf);
+		}
+
+		if (scaled_pixbuf != NULL) {
+			image = gtk_image_new_from_pixbuf (scaled_pixbuf);
+			g_object_unref (scaled_pixbuf);
+		}
+		
 		setup_menuitem (menuitem, 
-				gtk_image_new_from_file (pixmap_path),
+				image,
 				_("Panel"));
 
 		g_free (pixmap_path);
@@ -509,12 +563,13 @@ panel_applet_create_menu (AppletInfo *info,
 	GtkWidget *menuitem;
 	GList     *l;
 
-	info->menu = gtk_menu_new ();
+	info->menu = g_object_ref (gtk_menu_new ());
+	gtk_object_sink (GTK_OBJECT (info->menu));
 
 	if (!commie_mode) {
 		GtkWidget *image;
 
-		menuitem = gtk_menu_item_new ();
+		menuitem = gtk_image_menu_item_new ();
 
 		image = gtk_image_new_from_stock (GTK_STOCK_REMOVE,
 						  GTK_ICON_SIZE_MENU);
@@ -528,7 +583,7 @@ panel_applet_create_menu (AppletInfo *info,
 
 		gtk_menu_shell_append (GTK_MENU_SHELL (info->menu), menuitem);
 
-		menuitem = gtk_menu_item_new();
+		menuitem = gtk_image_menu_item_new();
 
 		/*
 		 * FIXME: should have a "Move" pixmap.
@@ -548,7 +603,7 @@ panel_applet_create_menu (AppletInfo *info,
 	gtk_menu_shell_append (GTK_MENU_SHELL (info->menu), menuitem);
 
 	if (info->user_menu) {
-		menuitem = gtk_menu_item_new ();
+		menuitem = gtk_image_menu_item_new ();
 
 		gtk_menu_shell_append (GTK_MENU_SHELL (info->menu), menuitem);
 
@@ -582,9 +637,9 @@ applet_menu_set_data_recursively (GtkMenu     *menu,
 	GList *children;
 	GList *l;
 
-	gtk_object_set_data (GTK_OBJECT (menu), key, data);
+	g_object_set_data (G_OBJECT (menu), key, data);
 
-	children = gtk_container_children (GTK_CONTAINER (menu));
+	children = gtk_container_get_children (GTK_CONTAINER (menu));
 
 	for (l = children; l; l = l->next) {
 		GtkWidget *submenu = GTK_MENU_ITEM (l->data)->submenu;
@@ -711,6 +766,9 @@ applet_destroy (GtkWidget *w, AppletInfo *info)
 	info->data_destroy = NULL;
 	info->data = NULL;
 
+	g_free (info->gconf_key);
+	info->gconf_key = NULL;
+
 	/*free the user menu*/
 	for(li = info->user_menu; li != NULL; li = g_list_next(li)) {
 		AppletUserMenu *umenu = li->data;
@@ -730,6 +788,253 @@ applet_destroy (GtkWidget *w, AppletInfo *info)
 	info->user_menu = NULL;
 }
 
+static char *
+panel_applet_get_full_gconf_key (AppletType   type,
+				 const gchar *profile,
+				 const gchar *object_id,
+				 const gchar *key,
+				 gboolean     use_default)
+{
+	char *retval = NULL;
+
+	if (type == APPLET_BONOBO)
+		retval = use_default ? panel_gconf_applets_default_profile_get_full_key (profile, object_id, key) :
+			panel_gconf_applets_profile_get_full_key (profile, object_id, key);
+	else
+		retval = use_default ? panel_gconf_objects_default_profile_get_full_key (profile, object_id, key) :
+			panel_gconf_objects_profile_get_full_key (profile, object_id, key);
+
+	return retval;
+}
+
+static void
+panel_applet_load_from_unique_id (AppletType   type,
+				  GConfClient *gconf_client,
+				  const char  *profile,
+				  const char  *unique_id,
+				  gboolean     use_default)
+{
+	PanelWidget *panel_widget;
+	AppletType   applet_type;
+	char        *type_string;
+	char        *temp_key;
+	char        *panel_id;
+	int          position;
+	gboolean     right_stick;
+
+	temp_key = panel_applet_get_full_gconf_key (type, profile, unique_id, "object-type", use_default);
+	type_string = gconf_client_get_string (gconf_client, temp_key, NULL);
+	g_free (temp_key);
+
+	if (!gconf_string_to_enum (object_type_enum_map, type_string, (int *) &applet_type)) {
+		g_free (type_string);
+		g_warning ("Unkown applet type %s from %s", type_string, temp_key);
+		return;
+	}
+	
+	g_free (type_string);
+
+	temp_key = panel_applet_get_full_gconf_key (type, profile, unique_id, "position", use_default);
+	position = gconf_client_get_int (gconf_client, temp_key, NULL);
+	g_free (temp_key);
+	
+	temp_key = panel_applet_get_full_gconf_key (type, profile, unique_id, "panel-id", use_default);
+	panel_id = gconf_client_get_string (gconf_client, temp_key, NULL);
+	g_free (temp_key);
+
+	temp_key = panel_applet_get_full_gconf_key (type, profile, unique_id, "panel-right-stick", use_default);
+	right_stick = gconf_client_get_bool (gconf_client, temp_key, NULL);
+	g_free (temp_key);
+
+	panel_widget = panel_widget_get_by_id (panel_id);
+	g_free (panel_id);
+
+	/*
+	 * A hack from the old panel to make sure the applet is on the right
+	 */
+	position += right_stick ? G_MAXINT/2 : 0;
+
+	switch (applet_type) {
+	case APPLET_BONOBO:
+		panel_applet_frame_load_from_gconf (panel_widget, position, unique_id, use_default);
+		break;
+	case APPLET_DRAWER:
+		drawer_load_from_gconf (panel_widget, position, unique_id, use_default);
+		break;
+	case APPLET_SWALLOW:
+		swallow_load_from_gconf (panel_widget, position, unique_id, use_default);
+		break;
+	case APPLET_MENU:
+		menu_load_from_gconf (panel_widget, position, unique_id, use_default);
+		break;
+	case APPLET_LAUNCHER:
+		launcher_load_from_gconf (panel_widget, position, unique_id, use_default);
+		break;
+	case APPLET_LOGOUT:
+		load_logout_applet (panel_widget, position, TRUE, use_default);
+		break;
+	case APPLET_LOCK:
+		load_lock_applet (panel_widget, position, TRUE, use_default);
+		break;
+	case APPLET_STATUS:
+		load_status_applet (panel_widget, position, TRUE, use_default);
+		break;
+	default:
+		break;
+	}
+}
+
+static void
+panel_applet_load_list (AppletType   type,
+			gboolean    use_default)
+{
+	GConfClient *client;
+	GSList      *id_list, *l;
+	char        *profile;
+	char        *temp_key = NULL;
+	
+	client = panel_gconf_get_client ();
+
+	if (use_default)
+		profile = PANEL_GCONF_DEFAULT_PROFILE;
+	else 
+		profile = session_get_current_profile ();
+
+	if (type == APPLET_BONOBO)
+		temp_key = use_default ? panel_gconf_general_default_profile_get_full_key (profile, "applet-id-list") :
+			panel_gconf_general_profile_get_full_key (profile, "applet-id-list");
+	else
+        	temp_key = use_default ? panel_gconf_general_default_profile_get_full_key (profile, "object-id-list") :
+			panel_gconf_general_profile_get_full_key (profile, "object-id-list");
+
+        id_list = gconf_client_get_list (panel_gconf_get_client (), temp_key, GCONF_VALUE_STRING, NULL);
+
+        for (l = id_list; l; l = l->next)
+		panel_applet_load_from_unique_id (type, client, profile, (char *) l->data, use_default);
+
+        g_free (temp_key);
+        g_slist_foreach (id_list, (GFunc) g_free, NULL);
+        g_slist_free (id_list);
+}
+
+void
+panel_applet_load_applets_from_gconf (void)
+{
+      	char     *profile_key;
+	gboolean  use_default = FALSE;
+
+	profile_key = g_strdup_printf ("/apps/panel/profiles/%s/objects", session_get_current_profile ());
+
+	if (panel_gconf_dir_exists (profile_key) == FALSE)
+		use_default = TRUE;
+	      
+	g_free (profile_key);
+
+	panel_applet_load_list (APPLET_BONOBO, use_default);
+	panel_applet_load_list (APPLET_EMPTY, use_default);
+}
+
+void
+panel_applet_save_position (AppletInfo *applet_info,
+			    const char *gconf_key)
+{
+	GConfClient *client;
+	char        *profile;
+	char        *temp_key;
+
+	client  = panel_gconf_get_client ();
+	profile = session_get_current_profile ();
+
+	temp_key = panel_applet_get_full_gconf_key (applet_info->type, profile, gconf_key, "position", FALSE);
+	gconf_client_set_int (client, temp_key, panel_applet_get_position (applet_info), NULL);
+	g_free (temp_key);
+
+	temp_key = panel_applet_get_full_gconf_key (applet_info->type, profile, gconf_key, "panel-id", FALSE);
+	gconf_client_set_string (client, temp_key, panel_applet_get_panel_id (applet_info), NULL);
+	g_free (temp_key);
+
+	temp_key = panel_applet_get_full_gconf_key (applet_info->type, profile, gconf_key, "right-stick", FALSE);
+	gconf_client_set_bool (client, temp_key, panel_applet_get_right_stick (applet_info), NULL);
+	g_free (temp_key);
+
+	gconf_client_suggest_sync (client, NULL);
+}
+
+void
+panel_applet_save_to_gconf (AppletInfo *applet_info)
+{
+	GConfClient *client;
+	char        *profile;
+	char        *temp_key;
+	GSList      *id_list, *l;
+
+	client  = panel_gconf_get_client ();
+	profile = session_get_current_profile ();
+
+	if (applet_info->type == APPLET_BONOBO)
+		temp_key = panel_gconf_general_profile_get_full_key (profile, "applet-id-list");
+	else
+		temp_key = panel_gconf_general_profile_get_full_key (profile, "object-id-list");
+
+	id_list = gconf_client_get_list (client, temp_key, GCONF_VALUE_STRING, NULL);
+
+	if (!applet_info->gconf_key)
+		applet_info->gconf_key = gconf_unique_key ();
+
+	for (l = id_list; l; l = l->next)
+		if (!strcmp (applet_info->gconf_key, (char *) l->data))
+			break;
+
+	if (!l) {
+		id_list = g_slist_prepend (id_list, g_strdup (applet_info->gconf_key));
+
+		gconf_client_set_list (client, temp_key, GCONF_VALUE_STRING, id_list, NULL);
+	}
+
+	g_free (temp_key);
+	g_slist_foreach (id_list, (GFunc) g_free, NULL);
+	g_slist_free (id_list);
+
+	temp_key = panel_applet_get_full_gconf_key (applet_info->type, profile,
+						    applet_info->gconf_key, "object-type", FALSE);
+	gconf_client_set_string (client, temp_key,
+				 gconf_enum_to_string (object_type_enum_map, applet_info->type),
+				 NULL);
+	g_free (temp_key);
+
+	panel_applet_save_position (applet_info, applet_info->gconf_key);
+
+	switch (applet_info->type) {
+	case APPLET_BONOBO:
+		panel_applet_frame_save_to_gconf (PANEL_APPLET_FRAME (applet_info->data),
+						  applet_info->gconf_key);
+		break;
+	case APPLET_DRAWER:
+		drawer_save_to_gconf ((Drawer *) applet_info->data,
+				      applet_info->gconf_key);
+		break;
+	case APPLET_SWALLOW:
+		swallow_save_to_gconf ((Swallow *) applet_info->data,
+				       applet_info->gconf_key);
+		break;
+	case APPLET_MENU:
+		menu_save_to_gconf ((Menu *) applet_info->data,
+				    applet_info->gconf_key);
+		break;
+	case APPLET_LAUNCHER:
+		launcher_save_to_gconf ((Launcher *) applet_info->data,
+					applet_info->gconf_key);
+		break;
+	case APPLET_LOGOUT: /* no object specific stuff to save */
+	case APPLET_LOCK:
+	case APPLET_STATUS:
+	default:
+		break;
+	}
+
+	gconf_client_suggest_sync (client, NULL);
+}
+
 AppletInfo *
 panel_applet_register (GtkWidget      *applet,
 		       gpointer        data,
@@ -737,11 +1042,13 @@ panel_applet_register (GtkWidget      *applet,
 		       PanelWidget    *panel,
 		       gint            pos,
 		       gboolean        exactpos,
-		       AppletType      type)
+		       AppletType      type,
+		       const char     *gconf_key)
 {
 	AppletInfo *info;
 	int newpos;
 	gboolean insert_at_pos;
+	gboolean expand_major = FALSE, expand_minor = FALSE;
 	
 	g_return_val_if_fail (applet && panel, NULL);
 
@@ -761,21 +1068,27 @@ panel_applet_register (GtkWidget      *applet,
 	info->data_destroy = data_destroy;
 	info->user_menu = NULL;
 
-	gtk_object_set_data (GTK_OBJECT (applet), "applet_info", info);
+	g_object_set_data (G_OBJECT (applet), "applet_info", info);
 
 	if (type == APPLET_DRAWER) {
 		Drawer *drawer = data;
 		PanelWidget *assoc_panel =
 			PANEL_WIDGET (BASEP_WIDGET (drawer->drawer)->panel);
 
-		gtk_object_set_data (GTK_OBJECT (applet),
-				     PANEL_APPLET_ASSOC_PANEL_KEY, assoc_panel);
+		g_object_set_data (G_OBJECT (applet),
+				   PANEL_APPLET_ASSOC_PANEL_KEY, assoc_panel);
 		assoc_panel->master_widget = applet;
 	}
 
-	gtk_object_set_data (GTK_OBJECT (applet),
-			     PANEL_APPLET_FORBIDDEN_PANELS, NULL);
-		
+	g_object_set_data (G_OBJECT (applet),
+			   PANEL_APPLET_FORBIDDEN_PANELS, NULL);
+
+	if (type == APPLET_BONOBO) {
+		panel_applet_frame_get_expand_flags (PANEL_APPLET_FRAME (applet),
+						     &expand_major,
+						     &expand_minor);
+	}
+	
 	applets = g_slist_append (applets, info);
 
 	applet_count++;
@@ -789,18 +1102,21 @@ panel_applet_register (GtkWidget      *applet,
 		insert_at_pos = FALSE;
 	} else {
 		newpos = 0;
-		insert_at_pos = TRUE;
+		insert_at_pos = FALSE;
 	}
 	/* if exact pos is on then insert at that precise location */
 	if (exactpos)
 		insert_at_pos = TRUE;
 
+
+	
+
 	if (panel_widget_add_full (panel, applet, newpos, TRUE,
-				   insert_at_pos)==-1) {
+				   insert_at_pos, expand_major, expand_minor)==-1) {
 		GSList *list;
 		for(list = panels; list != NULL; list = g_slist_next(list))
 			if (panel_widget_add_full (panel, applet, 0,
-						   TRUE, TRUE)!=-1)
+						   TRUE, TRUE, expand_major, expand_minor)!=-1)
 				break;
 		if(!list) {
 			/*can't put it anywhere, clean up*/
@@ -838,5 +1154,52 @@ panel_applet_register (GtkWidget      *applet,
 	size_change (info, panel);
 	back_change (info, panel);
 
+	if (gconf_key)
+		info->gconf_key = g_strdup (gconf_key);
+
+	panel_applet_save_to_gconf (info);
+
 	return info;
 }
+
+int
+panel_applet_get_position (AppletInfo *applet)
+{
+	AppletData *applet_data;
+
+	g_return_val_if_fail (applet, 0);
+	g_return_val_if_fail (G_IS_OBJECT (applet->widget), 0);
+
+	applet_data = g_object_get_data (G_OBJECT (applet->widget), PANEL_APPLET_DATA);
+
+	return applet_data->pos;
+}
+
+gchar *
+panel_applet_get_panel_id (AppletInfo *applet)
+{
+	PanelWidget *panel;
+
+	g_return_val_if_fail (applet, NULL);
+	g_return_val_if_fail (G_IS_OBJECT (applet->widget), NULL);
+
+	panel = PANEL_WIDGET (applet->widget->parent);
+
+	return panel->unique_id;
+}
+
+/*
+ * WTF ?
+ */
+gboolean
+panel_applet_get_right_stick (AppletInfo *applet)
+{
+	PanelWidget *panel;
+
+	g_return_val_if_fail (applet, FALSE);
+	g_return_val_if_fail (G_IS_OBJECT (applet->widget), FALSE);
+
+	panel = PANEL_WIDGET (applet->widget->parent);
+
+	return panel_widget_is_applet_stuck (panel, applet->widget);
+}
