@@ -12,16 +12,12 @@ G_BEGIN_DECLS
 #define DRAWER_ID   "Drawer"
 #define LOGOUT_ID   "Logout"
 #define SWALLOW_ID  "Swallow"
-#define EXTERN_ID   "Extern"
 #define LAUNCHER_ID "Launcher"
 #define LOCK_ID     "Lock"
 #define STATUS_ID   "Status"
-#define RUN_ID      "Run"
+#define BONOBO_ID   "Bonobo"
 
 typedef enum {
-	APPLET_EXTERN,
-	APPLET_EXTERN_RESERVED,
-	APPLET_EXTERN_PENDING,
 	APPLET_DRAWER,
 	APPLET_MENU,
 	APPLET_LOGOUT,
@@ -30,7 +26,6 @@ typedef enum {
 	APPLET_EMPTY,
 	APPLET_LOCK,
 	APPLET_STATUS,
-	APPLET_RUN,
 	APPLET_BONOBO
 } AppletType;
 
@@ -49,6 +44,8 @@ typedef struct {
 
 	gpointer        data;
 	GDestroyNotify  data_destroy;
+
+	char           *gconf_key;
 } AppletInfo;
 
 typedef struct {
@@ -69,7 +66,8 @@ AppletInfo *panel_applet_register    (GtkWidget      *applet,
 				      PanelWidget    *panel,
 				      gint            pos,
 				      gboolean        exactpos,
-				      AppletType      type);
+				      AppletType      type,
+				      const char     *gconf_key);
 
 void        panel_applet_clean       (AppletInfo *info);
 
@@ -89,9 +87,14 @@ AppletUserMenu *panel_applet_get_callback    (GList       *user_menu,
 					      const gchar *name);
 
 
-void        panel_applet_callback_set_sensitive (AppletInfo *info,
-						 const char *callback_name,
-						 gint        sensitive);
+void        panel_applet_load_applets_from_gconf (void);
+void        panel_applet_save_to_gconf           (AppletInfo *applet_info);
+void        panel_applet_save_position           (AppletInfo *applet_info,
+						  const char *gconf_key);
+
+int         panel_applet_get_position    (AppletInfo *applet);
+gchar      *panel_applet_get_panel_id    (AppletInfo *applet);
+gboolean    panel_applet_get_right_stick (AppletInfo *applet);
 
 G_END_DECLS
 
