@@ -17,7 +17,7 @@
 #include "panel-widget.h"
 #include "basep-widget.h"
 #include "panel-util.h"
-#include "panel_config_global.h"
+#include "panel-config-global.h"
 #include "foobar-widget.h"
 #include "drawer-widget.h"
 #include "border-widget.h"
@@ -85,6 +85,9 @@ enum {
         PROP_HIDEBUTTON_PIXMAPS_ENABLED,
 };
 
+#define PANEL_SLOW_STEP_SIZE 10
+#define PANEL_MEDIUM_STEP_SIZE 60
+#define PANEL_FAST_STEP_SIZE 80
 
 /************************
  widget core
@@ -122,7 +125,7 @@ basep_widget_class_init (BasePWidgetClass *klass)
 	GtkObjectClass *gtk_object_class = GTK_OBJECT_CLASS (klass);
 	GtkWidgetClass *widget_class = GTK_WIDGET_CLASS (klass);
 	
-	basep_widget_parent_class = gtk_type_class (gtk_window_get_type ());
+	basep_widget_parent_class = g_type_class_ref (gtk_window_get_type ());
 
 	klass->mode_change = basep_widget_mode_change;
 	klass->state_change = basep_widget_state_change;
@@ -529,11 +532,39 @@ basep_widget_mode_change (BasePWidget *basep, BasePMode mode)
 		basep_border_queue_recalc (basep->screen);
 }
 
+static void
+set_tip (GtkWidget *widget, gboolean showing) 
+{
+	if (showing) {
+		gtk_tooltips_set_tip (panel_tooltips, widget,
+		      _("Hide this panel"), NULL);
+	} else {
+		gtk_tooltips_set_tip (panel_tooltips, widget,
+		      _("Show this panel"), NULL);
+	}
+}
+
 static void
 basep_widget_state_change (BasePWidget *basep, BasePState state)
 {
 	if (BORDER_IS_WIDGET (basep))
 		basep_border_queue_recalc (basep->screen);
+	/*
+	 * If the state is changed to SHOWN or HIDDEN_LEFT or HIDDEN_RIGHT
+	 * we update the tooltips on the buttons
+	 */
+	if (state == BASEP_SHOWN) {
+		set_tip (basep->hidebutton_w, TRUE);
+		set_tip (basep->hidebutton_n, TRUE);
+		set_tip (basep->hidebutton_s, TRUE);
+		set_tip (basep->hidebutton_e, TRUE);
+	} else if (state == BASEP_HIDDEN_LEFT) {
+		set_tip (basep->hidebutton_e, FALSE);
+		set_tip (basep->hidebutton_s, FALSE);
+	} else if (state == BASEP_HIDDEN_RIGHT) {
+		set_tip (basep->hidebutton_n, FALSE);
+		set_tip (basep->hidebutton_w, FALSE);
+	}
 }
 
 static void
@@ -644,7 +675,7 @@ basep_pos_class_init (BasePPosClass *klass)
 {
 	/*GtkObjectClass *object_class = GTK_OBJECT_CLASS(klass);*/
 
-	basep_pos_parent_class = gtk_type_class(gtk_object_get_type ());
+	basep_pos_parent_class = g_type_class_ref (gtk_object_get_type ());
 
 	klass->get_hide_size = basep_pos_get_hide_size;
 	klass->get_hide_pos = basep_pos_get_hide_pos;
@@ -783,17 +814,32 @@ move_step(int src, int dest, long start_time, long end_time, long cur_time)
 
 void
 basep_widget_do_hiding(BasePWidget *basep, PanelOrient hide_orient,
-		       int leftover, int step)
+		       int leftover, PanelSpeed animation_step)
 {
 	GtkWidget *wid;
 	int ox,oy,ow,oh;
 	int x,y,w,h;
 	int dx,dy,dw,dh;
 	int diff;
+	int step;
 	
 	g_return_if_fail(basep != NULL);
 	g_return_if_fail(BASEP_IS_WIDGET(basep));
 
+	switch (animation_step) {
+		case PANEL_SPEED_SLOW:
+				step = PANEL_SLOW_STEP_SIZE;
+				break;
+		case PANEL_SPEED_MEDIUM:
+				step = PANEL_MEDIUM_STEP_SIZE;
+				break;
+		case PANEL_SPEED_FAST:
+				step = PANEL_FAST_STEP_SIZE;
+				break;
+		default:
+				step = PANEL_MEDIUM_STEP_SIZE; 
+	}
+
 #ifdef BASEP_WIDGET_DEBUG
 	g_warning ("do_hiding with step %d", step);
 #endif
@@ -847,7 +893,7 @@ basep_widget_do_hiding(BasePWidget *basep, PanelOrient hide_orient,
 		break;
 	}
 
-	if(!global_config.disable_animations && step != 0) {
+	if(global_config.enable_animations && step != 0) {
 		GTimeVal tval;
 		long start_secs;
 		long start_time;
@@ -891,20 +937,35 @@ basep_widget_do_hiding(BasePWidget *basep, PanelOrient hide_orient,
 	}
 	
 	gtk_widget_queue_resize(wid);
-	gtk_widget_draw(basep->table, NULL);
+	gtk_widget_queue_draw (basep->table);
 }
 
 void
 basep_widget_do_showing(BasePWidget *basep, PanelOrient hide_orient,
-			int leftover, int step)
+			int leftover, PanelSpeed animation_step)
 {
 	GtkWidget *wid;
 	int x,y, dx,dy, ox,oy;
 	int w,h, dw,dh, ow,oh;
 	int diff;
+	int step;
 
 	g_return_if_fail(basep != NULL);
 	g_return_if_fail(BASEP_IS_WIDGET(basep));
+	
+	switch (animation_step) {
+		case PANEL_SPEED_MEDIUM:
+				step = PANEL_MEDIUM_STEP_SIZE;
+				break;
+		case PANEL_SPEED_SLOW:
+				step = PANEL_SLOW_STEP_SIZE;
+				break;
+		case PANEL_SPEED_FAST:
+				step = PANEL_FAST_STEP_SIZE;
+				break;
+		default:
+				step = PANEL_MEDIUM_STEP_SIZE;
+	}
 
 #ifdef BASEP_WIDGET_DEBUG
 		g_warning ("do_showing with step %d", step);
@@ -954,7 +1015,7 @@ basep_widget_do_showing(BasePWidget *basep, PanelOrient hide_orient,
 		break;
 	}
 	
-	if(!global_config.disable_animations && step != 0) {
+	if(global_config.enable_animations && step != 0) {
 		int i;
 		GTimeVal tval;
 		long start_secs;
@@ -997,10 +1058,18 @@ basep_widget_do_showing(BasePWidget *basep, PanelOrient hide_orient,
 			
 			/*drawing the entire table flickers, so don't
 			  do it often*/
-			if(i++%10)
-				gtk_widget_draw(basep->panel, NULL);
-			else
-				gtk_widget_draw(basep->table, NULL);
+			/* FIXME: does it still flicker? */
+			if (i++ % 10) {
+				/* FIXME: is invalidation needed, and if so it should only be
+				 * partial */
+				gdk_window_invalidate_rect (basep->panel->window,
+							    NULL, TRUE);
+				gdk_window_process_updates (basep->panel->window, TRUE);
+			} else {
+				gdk_window_invalidate_rect (GTK_WIDGET (basep)->window,
+							    NULL, TRUE);
+				gdk_window_process_updates (GTK_WIDGET (basep)->window, TRUE);
+			}
 			gdk_flush();
 			g_usleep (1000);
 		}
@@ -1011,8 +1080,8 @@ basep_widget_do_showing(BasePWidget *basep, PanelOrient hide_orient,
 		basep_widget_set_ebox_orient(basep, -1);
 	}
 	
-	gtk_widget_draw(basep->table, NULL);
-	gtk_widget_queue_resize(wid);
+	gtk_widget_queue_draw (basep->table);
+	gtk_widget_queue_resize (wid);
 }
 
 
@@ -1026,11 +1095,11 @@ make_hidebutton(BasePWidget *basep,
 	char *pixmap_name;
 
 	w = gtk_button_new();
-	GTK_WIDGET_UNSET_FLAGS(w, GTK_CAN_DEFAULT|GTK_CAN_FOCUS);
+	GTK_WIDGET_UNSET_FLAGS(w, GTK_CAN_DEFAULT);
 	if(horizontal)
-		gtk_widget_set_usize(w, 0, PANEL_MINIMUM_WIDTH);
+		gtk_widget_set_size_request (w, -1, PANEL_MINIMUM_WIDTH);
 	else
-		gtk_widget_set_usize(w, PANEL_MINIMUM_WIDTH, 0);
+		gtk_widget_set_size_request (w, PANEL_MINIMUM_WIDTH, -1);
 
 	pixmap_name = gnome_program_locate_file (NULL, GNOME_FILE_DOMAIN_PIXMAP, 
 						 pixmaparrow, TRUE, NULL);
@@ -1043,13 +1112,11 @@ make_hidebutton(BasePWidget *basep,
 	}
 	gtk_widget_show(pixmap);
 
-	gtk_container_add(GTK_CONTAINER(w),pixmap);
-	gtk_object_set_user_data(GTK_OBJECT(w), pixmap);
-	gtk_object_set_data(GTK_OBJECT(w), "gnome_disable_sound_events",
-			    GINT_TO_POINTER(1));
+	gtk_container_add (GTK_CONTAINER (w), pixmap);
+	g_object_set_data (G_OBJECT (w), "gnome_disable_sound_events",
+			   GINT_TO_POINTER (TRUE));
 
-	gtk_tooltips_set_tip (panel_tooltips, w,
-			      _("Hide this panel"), NULL);
+	set_tip (w, TRUE);
 
 	return w;
 }
@@ -1249,28 +1316,27 @@ basep_widget_instance_init (BasePWidget *basep)
 }
 
 static void
-show_hidebutton_pixmap(GtkWidget *hidebutton, int show)
+show_hidebutton_pixmap (GtkWidget *hidebutton, gboolean show)
 {
-	GtkWidget *pixmap;
+	GtkWidget *pixmap = GTK_BIN (hidebutton)->child;
 
-	pixmap = gtk_object_get_user_data(GTK_OBJECT(hidebutton));
-
-	if (!pixmap) return;
+	if (pixmap == NULL)
+		return;
 
 	if (show)
-		gtk_widget_show(pixmap);
+		gtk_widget_show (pixmap);
 	else
-		gtk_widget_hide(pixmap);
+		gtk_widget_hide (pixmap);
 }
 
 static void
-basep_widget_show_hidebutton_pixmaps(BasePWidget *basep)
+basep_widget_show_hidebutton_pixmaps (BasePWidget *basep)
 {
-	int show = basep->hidebutton_pixmaps_enabled;
-	show_hidebutton_pixmap(basep->hidebutton_n, show);
-	show_hidebutton_pixmap(basep->hidebutton_e, show);
-	show_hidebutton_pixmap(basep->hidebutton_w, show);
-	show_hidebutton_pixmap(basep->hidebutton_s, show);
+	gboolean show = basep->hidebutton_pixmaps_enabled;
+	show_hidebutton_pixmap (basep->hidebutton_n, show);
+	show_hidebutton_pixmap (basep->hidebutton_e, show);
+	show_hidebutton_pixmap (basep->hidebutton_w, show);
+	show_hidebutton_pixmap (basep->hidebutton_s, show);
 }
 
 void
@@ -1281,7 +1347,8 @@ basep_widget_update_winhints (BasePWidget *basep)
 	gtk_window_set_decorated (GTK_WINDOW (w), FALSE);
 	gtk_window_stick (GTK_WINDOW (w));
 
-	xstuff_set_wmspec_dock_hints (w->window);
+	xstuff_set_wmspec_dock_hints (w->window,
+				      (basep->mode == BASEP_AUTO_HIDE));
 
 	/* FIXME: non-compliance should be tested! */
 
@@ -1407,7 +1474,8 @@ basep_widget_west_clicked (GtkWidget *widget, gpointer data)
 }
 
 GtkWidget*
-basep_widget_construct (BasePWidget *basep,
+basep_widget_construct (gchar *panel_id,
+			BasePWidget *basep,
 			gboolean packed,
 			gboolean reverse_arrows,
 			int screen,
@@ -1420,19 +1488,20 @@ basep_widget_construct (BasePWidget *basep,
 			PanelBackType back_type,
 			char *back_pixmap,
 			gboolean fit_pixmap_bg,
-			gboolean strech_pixmap_bg,
+			gboolean stretch_pixmap_bg,
 			gboolean rotate_pixmap_bg,
 			GdkColor *back_color)
 {
 	BasePPosClass *klass = basep_widget_get_pos_class (basep);
 	int x = 0, y = 0;
-	basep->panel = panel_widget_new(packed,
+	basep->panel = panel_widget_new(panel_id,
+					packed,
 					orient,
 					sz,
 					back_type,
 					back_pixmap,
 					fit_pixmap_bg,
-					strech_pixmap_bg,
+					stretch_pixmap_bg,
 					rotate_pixmap_bg,
 					back_color);
 
@@ -1520,7 +1589,7 @@ basep_widget_construct (BasePWidget *basep,
 		klass->get_pos (basep, &x, &y, 
 				PANEL_MINIMUM_WIDTH,
 				PANEL_MINIMUM_WIDTH);
-	gtk_widget_set_uposition (GTK_WIDGET (basep), x, y);
+	gtk_window_move (GTK_WINDOW (basep), x, y);
 	
 	return GTK_WIDGET (basep);
 }
@@ -1537,7 +1606,7 @@ basep_widget_change_params (BasePWidget *basep,
 			    PanelBackType back_type,
 			    char *pixmap_name,
 			    gboolean fit_pixmap_bg,
-			    gboolean strech_pixmap_bg,
+			    gboolean stretch_pixmap_bg,
 			    gboolean rotate_pixmap_bg,
 			    GdkColor *back_color)
 {
@@ -1579,7 +1648,7 @@ basep_widget_change_params (BasePWidget *basep,
 				   back_type,
 				   pixmap_name,
 				   fit_pixmap_bg,
-				   strech_pixmap_bg,
+				   stretch_pixmap_bg,
 				   rotate_pixmap_bg,
 				   back_color);
 
@@ -1591,7 +1660,7 @@ basep_widget_change_params (BasePWidget *basep,
 	gtk_widget_queue_resize (GTK_WIDGET (basep));
 }
 
-#if 0
+#if FIXME 
 gboolean
 basep_widget_convert_to (BasePWidget *basep,
 			 PanelType type)
@@ -1642,12 +1711,13 @@ basep_widget_convert_to (BasePWidget *basep,
 #endif
 
 void
-basep_widget_enable_buttons_ (BasePWidget *basep, gboolean enabled)
+basep_widget_enable_buttons (BasePWidget *basep,
+			     gboolean     enabled)
 {
-	gtk_widget_set_sensitive(basep->hidebutton_n, enabled);
-	gtk_widget_set_sensitive(basep->hidebutton_e, enabled);
-	gtk_widget_set_sensitive(basep->hidebutton_w, enabled);
-	gtk_widget_set_sensitive(basep->hidebutton_s, enabled);
+	gtk_widget_set_sensitive (basep->hidebutton_n, enabled);
+	gtk_widget_set_sensitive (basep->hidebutton_e, enabled);
+	gtk_widget_set_sensitive (basep->hidebutton_w, enabled);
+	gtk_widget_set_sensitive (basep->hidebutton_s, enabled);
 }
 
 void
@@ -1720,6 +1790,22 @@ basep_widget_explicit_hide (BasePWidget *basep, BasePState state)
 					global_config.animation_speed);
 	}
 
+	/*
+	 * If the button being hidden has focus move it to the button which
+	 * remains shown.
+	 */
+	if (state == BASEP_HIDDEN_RIGHT) {
+		if (gtk_widget_is_focus (basep->hidebutton_e))
+			gtk_widget_grab_focus (basep->hidebutton_w);
+		else if (gtk_widget_is_focus (basep->hidebutton_s))
+			gtk_widget_grab_focus (basep->hidebutton_n);
+	} else if (state == BASEP_HIDDEN_LEFT) {
+		if (gtk_widget_is_focus (basep->hidebutton_w))
+			gtk_widget_grab_focus (basep->hidebutton_e);
+		else if (gtk_widget_is_focus (basep->hidebutton_n))
+			gtk_widget_grab_focus (basep->hidebutton_s);
+	}
+
 	basep->state = state;
 	basep_widget_update_winhints (basep);
 }
@@ -1895,7 +1981,7 @@ basep_widget_autohide (gpointer data)
 		return TRUE;
 
 	if (!gdk_pointer_is_grabbed ()) {
-		if (gdk_pointer_grab (GDK_ROOT_PARENT(), FALSE, 
+		if (gdk_pointer_grab (gdk_get_default_root_window (), FALSE, 
 				      0, NULL, NULL, GDK_CURRENT_TIME)
 		    != GrabSuccess) {
 			return TRUE;
@@ -2035,60 +2121,6 @@ basep_widget_get_applet_orient (BasePWidget *basep)
 	return klass->get_applet_orient(basep);
 }
 
-#if 0
-void
-basep_widget_get_hide_size (BasePWidget *basep,
-			    PanelOrient hide_orient,
-			    int *w, int *h)
-{
-	BasePPosClass *klass = basep_widget_get_pos_class(basep);
-
-	basep_widget_get_size(basep, w, h);
-
-	if (basep->state == BASEP_SHOWN) {
-		g_warning ("get_hide_size() called on shown BasePWidget");
-		return;
-	}
-	
-	g_return_if_fail (klass && klass->get_hide_size);
-	klass->get_hide_size (basep, hide_orient, w, h);
-}
-
-void
-basep_widget_get_hide_orient (BasePWidget *basep,
-			      PanelOrient *hide_orient)
-{
-	BasePPosClass *klass = basep_widget_get_pos_class(basep);
-
-	*hide_orient = -1;
-	if (basep->state == BASEP_SHOWN) {
-		g_warning ("get_hide_orient() called on shown BasePWidget");
-		return;
-	}
-	
-	g_return_if_fail (klass && klass->get_hide_size);
-	klass->get_hide_orient (basep, hide_orient);
-}
-
-void
-basep_widget_get_hide_pos (BasePWidget *basep,
-			   PanelOrient hide_orient,
-			   int *x, int *y)
-{
-	BasePPosClass *klass = basep_widget_get_pos_class(basep);
-	int w, h;
-
-	if (basep->state == BASEP_SHOWN) {
-		g_warning ("get_hide_pos() called on shown BasePWidget");
-		return;
-	}
-	
-	basep_widget_get_hide_size (basep, hide_orient, &w, &h);
-	g_return_if_fail (klass && klass->get_hide_size);
-	klass->get_hide_orient (basep, hide_orient);
-}
-#endif
-
 void
 basep_widget_get_size (BasePWidget *basep,
 		       int *w, int *h)
@@ -2170,22 +2202,6 @@ basep_widget_pre_convert_hook (BasePWidget *basep)
 	klass->pre_convert_hook (basep);
 }
 
-void
-basep_widget_set_state (BasePWidget *basep, BasePState state,
-			gboolean emit)
-{
-	if (basep->state == state)
-		return;
-
-	basep->state = state;
-	
-	if (emit)
-		g_signal_emit (G_OBJECT(basep),
-			       basep_widget_signals[STATE_CHANGE_SIGNAL],
-			       0, state);
-	panels_to_sync = TRUE;
-}
-
 void
 basep_widget_screen_change (BasePWidget *basep, int screen)
 {
@@ -2350,10 +2366,7 @@ basep_border_recalc (int screen)
 		sb->borders[i].right = 0;
 	}
 
-	/* if not avoiding collisions, keeping things at 0 is a safe bet */
-	if (global_config.avoid_collisions) {
-		basep_calculate_borders (screen);
-	}
+	basep_calculate_borders (screen);
 
 	sb->left = border_max (sb, BORDER_LEFT);
 	sb->right = border_max (sb, BORDER_RIGHT);
@@ -2411,7 +2424,7 @@ queue_recalc_handler (gpointer data)
 		basep_border_recalc (screen);
 	}
 
-	g_list_free (recalc_list);
+	g_list_free (list);
 
 	return FALSE;
 }
