@@ -5,21 +5,21 @@
 #include <gtk/gtktypeutils.h>
 #include "GNOME_Panel.h"
 
-typedef GNOME_PanelOrient PanelOrient;
+typedef GNOME_Vertigo_PanelOrient PanelOrient;
 
-#define PANEL_ORIENT_UP    GNOME_PANEL_ORIENT_UP
-#define PANEL_ORIENT_DOWN  GNOME_PANEL_ORIENT_DOWN
-#define PANEL_ORIENT_LEFT  GNOME_PANEL_ORIENT_LEFT
-#define PANEL_ORIENT_RIGHT GNOME_PANEL_ORIENT_RIGHT
+#define PANEL_ORIENT_UP    GNOME_Vertigo_PANEL_ORIENT_UP
+#define PANEL_ORIENT_DOWN  GNOME_Vertigo_PANEL_ORIENT_DOWN
+#define PANEL_ORIENT_LEFT  GNOME_Vertigo_PANEL_ORIENT_LEFT
+#define PANEL_ORIENT_RIGHT GNOME_Vertigo_PANEL_ORIENT_RIGHT
 
 typedef enum {
-	PANEL_SIZE_XX_SMALL = GNOME_PANEL_XX_SMALL,
-	PANEL_SIZE_X_SMALL  = GNOME_PANEL_X_SMALL,
-	PANEL_SIZE_SMALL    = GNOME_PANEL_SMALL,
-	PANEL_SIZE_MEDIUM   = GNOME_PANEL_MEDIUM,
-	PANEL_SIZE_LARGE    = GNOME_PANEL_LARGE,
-	PANEL_SIZE_X_LARGE  = GNOME_PANEL_X_LARGE,
-	PANEL_SIZE_XX_LARGE = GNOME_PANEL_XX_LARGE 
+	PANEL_SIZE_XX_SMALL = GNOME_Vertigo_PANEL_XX_SMALL,
+	PANEL_SIZE_X_SMALL  = GNOME_Vertigo_PANEL_X_SMALL,
+	PANEL_SIZE_SMALL    = GNOME_Vertigo_PANEL_SMALL,
+	PANEL_SIZE_MEDIUM   = GNOME_Vertigo_PANEL_MEDIUM,
+	PANEL_SIZE_LARGE    = GNOME_Vertigo_PANEL_LARGE,
+	PANEL_SIZE_X_LARGE  = GNOME_Vertigo_PANEL_X_LARGE,
+	PANEL_SIZE_XX_LARGE = GNOME_Vertigo_PANEL_XX_LARGE 
 } PanelSize;
 
 typedef GNOME_Panel_BackType PanelBackType;
@@ -60,4 +60,10 @@ typedef enum {
 	BORDER_LEFT
 } BorderEdge;
 
+typedef enum {
+	PANEL_SPEED_SLOW,
+	PANEL_SPEED_MEDIUM,
+	PANEL_SPEED_FAST
+} PanelSpeed;
+
 #endif
