@@ -60,17 +60,16 @@ struct _PanelWidget
 {
 	GtkFixed		fixed;
 
-	guint32			unique_id;
+	gchar 			*unique_id;
 	
 	GList			*applet_list;
-	GList			*no_window_applet_list;
 
 	int			size;
 	GtkOrientation		orient;
 	int			sz;
 	gboolean		fit_pixmap_bg;  /* fit pixmap while keeping
 						   ratio*/
-	gboolean		strech_pixmap_bg; /*strech pixmap to the size
+	gboolean		stretch_pixmap_bg; /*stretch pixmap to the size
 						    of the panel */
 	gboolean		rotate_pixmap_bg; /*rotate pixmap on vertical
 						    panels */
@@ -123,21 +122,20 @@ struct _PanelWidgetClass
 			      PanelBackType type,
 			      char *pixmap,
 			      GdkColor *color);
-	void (* applet_draw) (PanelWidget *panel,
-			      GtkWidget *applet);
 	void (* applet_about_to_die) (PanelWidget *panel,
 				      GtkWidget *applet);
 };
 
 GType		panel_widget_get_type		(void) G_GNUC_CONST;
 
-GtkWidget *	panel_widget_new		(gboolean packed,
+GtkWidget *	panel_widget_new		(gchar *panel_id,
+						 gboolean packed,
 						 GtkOrientation orient,
 						 int sz,
 						 PanelBackType back_type,
 						 char *back_pixmap,
 						 gboolean fit_pixmap_bg,
-						 gboolean strech_pixmap_bg,
+						 gboolean stretch_pixmap_bg,
 						 gboolean rotate_pixmap_bg,
 						 GdkColor *back_color);
 /*add an applet to the panel, preferably at position pos, if insert_at_pos
@@ -151,9 +149,10 @@ int		panel_widget_add_full		(PanelWidget *panel,
 						 gboolean expand_minor);
 #define panel_widget_add(panel,applet,pos) (panel_widget_add_full(panel,applet,pos,TRUE,FALSE,FALSE,FALSE))
 
-PanelWidget *	panel_widget_get_by_id		(gint32 id);
+PanelWidget *	panel_widget_get_by_id		(gchar *id);
 void		panel_widget_set_id		(PanelWidget *panel,
-						 gint32 id);
+						 const char *id);
+void		panel_widget_set_new_id		(PanelWidget *panel);
 
 /*needs to be called for drawers after add*/
 void		panel_widget_add_forbidden	(PanelWidget *panel);
@@ -168,9 +167,6 @@ int		panel_widget_reparent		(PanelWidget *old_panel,
 						 PanelWidget *new_panel,
 						 GtkWidget *applet,
 						 int pos);
-/*return position of an applet*/
-int		panel_widget_get_pos		(PanelWidget *panel,
-						 GtkWidget *applet);
 
 /*get amount of free space around the applet (including the applet size),
   or return 0 on error or if the panel is packed*/
@@ -200,16 +196,12 @@ void		panel_widget_change_params	(PanelWidget *panel,
 						 PanelBackType back_type,
 						 char *pixmap_name,
 						 gboolean fit_pixmap_bg,
-						 gboolean strech_pixmap_bg,
+						 gboolean stretch_pixmap_bg,
 						 gboolean rotate_pixmap_bg,
 						 GdkColor *back_color);
 
-/* changing parameters (orient only) */
-void		panel_widget_change_orient	(PanelWidget *panel,
-						 GtkOrientation orient);
-
 void		panel_widget_set_back_pixmap	(PanelWidget *panel,
-						 char *file);
+						 const char *file);
 void		panel_widget_set_back_color	(PanelWidget *panel,
 						 GdkColor *color);
 
@@ -221,9 +213,6 @@ void		panel_widget_draw_icon		(PanelWidget *panel,
 						 ButtonWidget *applet);
 
 
-/*get the number of applets*/
-int		panel_widget_get_applet_count	(PanelWidget *panel);
-
 /*tells us if an applet is "stuck" on the right side*/
 int		panel_widget_is_applet_stuck	(PanelWidget *panel,
 						 GtkWidget *applet);
@@ -234,17 +223,6 @@ int		panel_widget_get_cursorloc	(PanelWidget *panel);
 gboolean	panel_widget_is_cursor		(PanelWidget *panel,
 						 int overlap);
 
-/* when we get color_only, we also optionally set r, g, b to the
-   color and w, and h to the area if the background is one color
-   only, otherwise normally return an rgb and set r, g, b to -1 */
-void panel_widget_get_applet_rgb_bg(PanelWidget *panel,
-				    GtkWidget *applet,
-				    guchar **rgb,
-				    int *w, int *h,
-				    int *rowstride,
-				    gboolean color_only,
-				    int *r, int *g, int *b);
-
 extern gboolean panel_applet_in_drag;
 
 G_END_DECLS
