@@ -5,28 +5,24 @@
  * Authors: George Lebl
  */
 #include <config.h>
-#include <gnome.h>
 #include <gdk/gdkx.h>
+#include <string.h>
 
 #include <X11/Xmd.h>
 #include <X11/Xlib.h>
 #include <X11/Xatom.h>
 
-#include "panel-include.h"
-#include "gnome-panel.h"
-#include "global-keys.h"
-#include "gwmh.h"
+/* Yes, yes I know, now bugger off ... */
+#define WNCK_I_KNOW_THIS_IS_UNSTABLE
+#include <libwnck/libwnck.h>
 
 #include "xstuff.h"
 #include "multiscreen-stuff.h"
-
-static GdkAtom KWM_MODULE = 0;
-static GdkAtom KWM_MODULE_DOCKWIN_ADD = 0;
-static GdkAtom KWM_MODULE_DOCKWIN_REMOVE = 0;
-static GdkAtom KWM_DOCKWINDOW = 0;
-static GdkAtom NAUTILUS_DESKTOP_WINDOW_ID = 0;
-static GdkAtom _NET_WM_DESKTOP = 0;
-static GdkAtom GNOME_PANEL_DESKTOP_AREA = 0;
+#include "basep-widget.h"
+#include "foobar-widget.h"
+#include "global-keys.h"
+#include "status.h"
+#include "swallow.h"
 
 extern GList *check_swallows;
 
@@ -36,18 +32,50 @@ extern GSList *panel_list;
 static void xstuff_setup_global_desktop_area (int left, int right,
 					      int top, int bottom);
 
+#define ATOM(name) xstuff_atom_intern(GDK_DISPLAY(),name)
+/* Once we have multiple display support we need to only use
+ * the below ones */
+
+#define ATOMD(display,name) xstuff_atom_intern(display,name)
+#define ATOMEV(event,name) xstuff_atom_intern(((XAnyEvent *)event)->display,name)
+#define ATOMGDK(win,name) xstuff_atom_intern(GDK_WINDOW_XDISPLAY(win),name)
+
+Atom
+xstuff_atom_intern (Display *display, const char *name)
+{
+	static GHashTable *cache = NULL;
+	char *key;
+	Atom atom;
+
+	if (cache == 0)
+		cache = g_hash_table_new (g_str_hash, g_str_equal);
+
+	key = g_strdup_printf ("%p %s", display, name);
+
+	atom = (Atom)g_hash_table_lookup (cache, key);
+	if (atom == 0) {
+		atom = XInternAtom (display, name, False);
+		g_hash_table_insert (cache, key, (gpointer)atom);
+	} else {
+		g_free (key);
+	}
+
+	return atom;
+}
+
 static void
 steal_statusspot(StatusSpot *ss, Window winid)
 {
 	GdkDragProtocol protocol;
 
-	gtk_socket_steal(GTK_SOCKET(ss->socket), winid);
+	gtk_socket_add_id (GTK_SOCKET (ss->socket), winid);
 	if (gdk_drag_get_protocol (winid, &protocol))
 		gtk_drag_dest_set_proxy (GTK_WIDGET (ss->socket),
 					 GTK_SOCKET(ss->socket)->plug_window,
 					 protocol, TRUE);
 }
 
+#ifdef FIXME
 static void
 try_adding_status(guint32 winid)
 {
@@ -59,8 +87,8 @@ try_adding_status(guint32 winid)
 
 	data = get_typed_property_data (GDK_DISPLAY (),
 					winid,
-					KWM_DOCKWINDOW,
-					KWM_DOCKWINDOW,
+					ATOM ("KWM_DOCKWINDOW"),
+					ATOM ("KWM_DOCKWINDOW"),
 					&size, 32);
 
 	if(data && *data) {
@@ -93,9 +121,12 @@ try_checking_swallows (GwmhTask *task)
 	}
 }
 
+#endif /* FIXME */
+
 void
 xstuff_go_through_client_list (void)
 {
+#ifdef FIXME
 	GList *li;
 
 	gdk_error_trap_push ();
@@ -112,84 +143,14 @@ xstuff_go_through_client_list (void)
 	}
 	gdk_flush();
 	gdk_error_trap_pop ();
-}
-
-static void
-redo_interface (void)
-{
-	GSList *li;
-	for (li = panel_list; li != NULL; li = li->next) {
-		PanelData *pd = li->data;
-		if (IS_BASEP_WIDGET (pd->panel))
-			basep_widget_redo_window (BASEP_WIDGET (pd->panel));
-		else if (IS_FOOBAR_WIDGET (pd->panel))
-			foobar_widget_redo_window (FOOBAR_WIDGET (pd->panel));
-	}
-}
-
-/* some deskguide code borrowed */
-static gboolean
-desk_notifier (gpointer func_data,
-	       GwmhDesk *desk,
-	       GwmhDeskInfoMask change_mask)
-{
-	if (change_mask & GWMH_DESK_INFO_BOOTUP)
-		redo_interface ();
-
-	/* we should maybe notice desk changes here */
-
-	return TRUE;
-}
-
-static gboolean
-task_notifier (gpointer func_data,
-	       GwmhTask *task,
-	       GwmhTaskNotifyType ntype,
-	       GwmhTaskInfoMask imask)
-{
-	/* skip own windows */
-	if (task->name != NULL &&
-	    strcmp (task->name, "panel") == 0)
-		return TRUE;
-
-	switch (ntype) {
-	case GWMH_NOTIFY_NEW:
-		try_adding_status (task->xwin);
-		/* fall through */
-	case GWMH_NOTIFY_INFO_CHANGED:
-		if (check_swallows != NULL)
-			try_checking_swallows (task);
-		break;
-	default:
-		break;
-	}
-
-	return TRUE;
+#endif
 }
 
 void
 xstuff_init (void)
 {
-	KWM_MODULE = gdk_atom_intern ("KWM_MODULE", FALSE);
-	KWM_MODULE_DOCKWIN_ADD =
-		gdk_atom_intern ("KWM_MODULE_DOCKWIN_ADD", FALSE);
-	KWM_MODULE_DOCKWIN_REMOVE =
-		gdk_atom_intern ("KWM_MODULE_DOCKWIN_REMOVE", FALSE);
-	KWM_DOCKWINDOW = gdk_atom_intern ("KWM_DOCKWINDOW", FALSE);
-	NAUTILUS_DESKTOP_WINDOW_ID =
-		gdk_atom_intern ("NAUTILUS_DESKTOP_WINDOW_ID", FALSE);
-	_NET_WM_DESKTOP =
-		gdk_atom_intern ("_NET_WM_DESKTOP", FALSE);
-	GNOME_PANEL_DESKTOP_AREA =
-		gdk_atom_intern ("GNOME_PANEL_DESKTOP_AREA", FALSE);
-
-	gwmh_init ();
-
-	gwmh_desk_notifier_add (desk_notifier, NULL);
-	gwmh_task_notifier_add (task_notifier, NULL);
-
 	/* setup the keys filter */
-	gdk_window_add_filter (GDK_ROOT_PARENT(),
+	gdk_window_add_filter (gdk_get_default_root_window (),
 			       panel_global_keys_filter,
 			       NULL);
 
@@ -213,15 +174,19 @@ xstuff_nautilus_desktop_present (void)
 	gdk_error_trap_push ();
 	data = get_typed_property_data (GDK_DISPLAY (),
 					GDK_ROOT_WINDOW (),
-					NAUTILUS_DESKTOP_WINDOW_ID,
+					ATOM ("NAUTILUS_DESKTOP_WINDOW_ID"),
 					XA_WINDOW,
 					&size, 32);
+
+        /* Get random property off the window to see if the window
+         * still exists.
+         */
 	if (data != NULL &&
 	    *data != 0) {
 		guint32 *desktop;
 		desktop = get_typed_property_data (GDK_DISPLAY (),
 						   *data,
-						   _NET_WM_DESKTOP,
+						   ATOM ("_NET_WM_DESKTOP"),
 						   XA_CARDINAL,
 						   &size, 32);
 		if (size > 0)
@@ -237,11 +202,18 @@ xstuff_nautilus_desktop_present (void)
 }
 
 void
-xstuff_set_simple_hint (GdkWindow *w, GdkAtom atom, long val)
+xstuff_set_simple_hint (GdkWindow *w, const char *name, long val)
 {
+	Atom atom = ATOMGDK (w, name);
+
 	gdk_error_trap_push ();
-	XChangeProperty (GDK_DISPLAY (), GDK_WINDOW_XWINDOW (w), atom, atom,
-			 32, PropModeReplace, (unsigned char*)&val, 1);
+
+	XChangeProperty (GDK_DISPLAY (),
+			 GDK_WINDOW_XWINDOW (w),
+			 atom, atom,
+			 32, PropModeReplace,
+			 (unsigned char*)&val, 1);
+
 	gdk_flush ();
 	gdk_error_trap_pop ();
 }
@@ -254,7 +226,8 @@ status_event_filter (GdkXEvent *gdk_xevent, GdkEvent *event, gpointer data)
 	xevent = (XEvent *)gdk_xevent;
 
 	if(xevent->type == ClientMessage) {
-		if(xevent->xclient.message_type == KWM_MODULE_DOCKWIN_ADD &&
+		if(xevent->xclient.message_type ==
+		   ATOMEV (xevent, "KWM_MODULE_DOCKWIN_ADD") &&
 		   !status_applet_get_ss(xevent->xclient.data.l[0])) {
 			Window w = xevent->xclient.data.l[0];
 			StatusSpot *ss;
@@ -262,7 +235,7 @@ status_event_filter (GdkXEvent *gdk_xevent, GdkEvent *event, gpointer data)
 			if (ss != NULL)
 				steal_statusspot (ss, w);
 		} else if(xevent->xclient.message_type ==
-			  KWM_MODULE_DOCKWIN_REMOVE) {
+			  ATOMEV (xevent, "KWM_MODULE_DOCKWIN_REMOVE")) {
 			StatusSpot *ss;
 			ss = status_applet_get_ss (xevent->xclient.data.l[0]);
 			if (ss != NULL)
@@ -276,11 +249,12 @@ status_event_filter (GdkXEvent *gdk_xevent, GdkEvent *event, gpointer data)
 void
 xstuff_setup_kde_dock_thingie (GdkWindow *w)
 {
-	xstuff_set_simple_hint (w, KWM_MODULE, 2);
+	xstuff_set_simple_hint (w, "KWM_MODULE", 2);
 	gdk_window_add_filter (w, status_event_filter, NULL);
-	send_client_message_1L (GDK_ROOT_WINDOW (), GDK_WINDOW_XWINDOW (w),
-				KWM_MODULE, SubstructureNotifyMask,
-				GDK_WINDOW_XWINDOW (w));
+	send_client_message_3L (GDK_ROOT_WINDOW (), GDK_WINDOW_XWINDOW (w),
+				ATOMGDK (w, "KWM_MODULE"),
+				SubstructureNotifyMask,
+				GDK_WINDOW_XWINDOW (w), 0, 0);
 }
 
 /* Stolen from deskguide */
@@ -382,11 +356,13 @@ get_typed_property_data (Display *xdisplay,
 
 /* sorta stolen from deskguide */
 gboolean
-send_client_message_1L (Window recipient,
+send_client_message_3L (Window recipient,
 			Window event_window,
 			Atom   message_type,
 			long   event_mask,
-			glong  long1)
+			long   long1,
+			long   long2,
+			long   long3)
 {
   XEvent xevent = { 0 };
 
@@ -407,11 +383,26 @@ send_client_message_1L (Window recipient,
 gboolean
 xstuff_is_compliant_wm (void)
 {
-	GwmhDesk *desk;
+	gpointer data;
+	int size;
 
-	desk = gwmh_desk_get_config ();
-
-	return desk->detected_gnome_wm;
+        /* FIXME this is totally broken; should be using
+         * gdk_net_wm_supports() on particular hints when we rely
+         * on those particular hints
+         */
+        
+	data = get_typed_property_data (GDK_DISPLAY (),
+					GDK_ROOT_WINDOW (),
+					ATOM ("_NET_SUPPORTED"),
+					XA_ATOM,
+					&size, 32);
+	if (data != NULL) {
+		/* Actually checks for some of these */
+		g_free (data);
+		return TRUE;
+	} else {
+		return FALSE;
+	}
 }
 
 void
@@ -419,13 +410,10 @@ xstuff_set_no_group_and_no_input (GdkWindow *win)
 {
 	XWMHints *old_wmhints;
 	XWMHints wmhints = {0};
-	static GdkAtom wm_client_leader_atom = GDK_NONE;
 
-	if (wm_client_leader_atom == GDK_NONE)
-		wm_client_leader_atom = gdk_atom_intern ("WM_CLIENT_LEADER",
-							 FALSE);
-
-	gdk_property_delete (win, wm_client_leader_atom);
+	XDeleteProperty (GDK_WINDOW_XDISPLAY (win),
+			 GDK_WINDOW_XWINDOW (win),
+			 ATOMGDK (win, "WM_CLIENT_LEADER"));
 
 	old_wmhints = XGetWMHints (GDK_DISPLAY (), GDK_WINDOW_XWINDOW (win));
 	/* General paranoia */
@@ -471,9 +459,12 @@ xstuff_setup_global_desktop_area (int left, int right, int top, int bottom)
 	vals[2] = top;
 	vals[3] = bottom;
 
-	XChangeProperty (GDK_DISPLAY (), GDK_ROOT_WINDOW (), 
-			 GNOME_PANEL_DESKTOP_AREA, XA_CARDINAL,
-			 32, PropModeReplace, (unsigned char*)vals, 4);
+	XChangeProperty (GDK_DISPLAY (),
+			 GDK_ROOT_WINDOW (), 
+			 ATOM ("GNOME_PANEL_DESKTOP_AREA"),
+			 XA_CARDINAL,
+			 32, PropModeReplace,
+			 (unsigned char *)vals, 4);
 
 	old_left = left;
 	old_right = right;
@@ -485,7 +476,6 @@ void
 xstuff_setup_desktop_area (int screen, int left, int right, int top, int bottom)
 {
 	char *screen_atom;
-	GdkAtom atom;
 	long vals[4];
 	static int screen_width = -1, screen_height = -1;
 
@@ -505,12 +495,14 @@ xstuff_setup_desktop_area (int screen, int left, int right, int top, int bottom)
 	 * root window, this should just set the GNOME_PANEL_DESKTOP_AREA */
 	screen_atom = g_strdup_printf ("GNOME_PANEL_DESKTOP_AREA_%d",
 				       screen);
-	atom = gdk_atom_intern (screen_atom, FALSE);
-	g_free (screen_atom);
+	XChangeProperty (GDK_DISPLAY (),
+			 GDK_ROOT_WINDOW (), 
+			 ATOM (screen_atom),
+			 XA_CARDINAL,
+			 32, PropModeReplace,
+			 (unsigned char *)vals, 4);
 
-	XChangeProperty (GDK_DISPLAY (), GDK_ROOT_WINDOW (), 
-			 atom, XA_CARDINAL,
-			 32, PropModeReplace, (unsigned char*)vals, 4);
+	g_free (screen_atom);
 
 	xstuff_setup_global_desktop_area
 		((multiscreen_x (screen)      == 0)             ? left   : -1,
@@ -529,66 +521,102 @@ xstuff_unsetup_desktop_area (void)
 {
 	int i;
 	char *screen_atom;
-	GdkAtom atom;
 
 	gdk_error_trap_push ();
 
-	XDeleteProperty (GDK_DISPLAY (), GDK_ROOT_WINDOW (),
-			 GNOME_PANEL_DESKTOP_AREA);
+	XDeleteProperty (GDK_DISPLAY (),
+			 GDK_ROOT_WINDOW (),
+			 ATOM ("GNOME_PANEL_DESKTOP_AREA"));
 
 	for (i = 0; i < multiscreen_screens (); i++) {
 		screen_atom =
 			g_strdup_printf ("GNOME_PANEL_DESKTOP_AREA_%d", i);
-		atom = gdk_atom_intern (screen_atom, FALSE);
-		g_free (screen_atom);
 
-		XDeleteProperty (GDK_DISPLAY (), GDK_ROOT_WINDOW (), atom);
+		XDeleteProperty (GDK_DISPLAY (),
+				 GDK_ROOT_WINDOW (),
+				 ATOM (screen_atom));
+
+		g_free (screen_atom);
 	}
 
 	gdk_flush ();
 	gdk_error_trap_pop ();
 }
 
+/* This is such a broken stupid function. */   
 void
 xstuff_set_pos_size (GdkWindow *window, int x, int y, int w, int h)
 {
-	Window win = GDK_WINDOW_XWINDOW (window);
-	XSizeHints size_hints;
+       Window win = GDK_WINDOW_XWINDOW (window);
+       XSizeHints size_hints;
+
+       /* Do not add USPosition / USSize here, fix the damn WM */
+       size_hints.flags = PPosition | PSize | PMaxSize | PMinSize;
+       size_hints.x = 0; /* window managers aren't supposed to and  */
+       size_hints.y = 0; /* don't use these fields */
+       size_hints.width = w;
+       size_hints.height = h;
+       size_hints.min_width = w;
+       size_hints.min_height = h;
+       size_hints.max_width = w;
+       size_hints.max_height = h;
   
-	size_hints.flags = USPosition | PPosition | USSize | PSize | PMaxSize | PMinSize;
-	size_hints.x = x;
-	size_hints.y = y;
-	size_hints.width = w;
-	size_hints.height = h;
-	size_hints.min_width = w;
-	size_hints.min_height = h;
-	size_hints.max_width = w;
-	size_hints.max_height = h;
-  
-	gdk_error_trap_push ();
+       gdk_error_trap_push ();
 
-	XSetWMNormalHints (GDK_DISPLAY (), win, &size_hints);
+       XSetWMNormalHints (GDK_DISPLAY (), win, &size_hints);
 
-	gdk_window_move_resize (window, x, y, w, h);
+       gdk_window_move_resize (window, x, y, w, h);
 
-	gdk_error_trap_pop ();
+       gdk_flush ();
+       gdk_error_trap_pop ();
 }
 
-
 void
-xstuff_set_wmspec_dock_hints (GdkWindow *window)
+xstuff_set_wmspec_dock_hints (GdkWindow *window,
+			      gboolean autohide)
 {
-        Atom atom;
+        Atom atoms[2] = { None, None };
         
-        atom = XInternAtom (gdk_display,
-                            "_NET_WM_WINDOW_TYPE_DOCK",
-                            False);
+	if (autohide) {
+		atoms[0] = ATOMGDK (window, "_GNOME_WINDOW_TYPE_AUTOHIDE_PANEL");
+		atoms[1] = ATOMGDK (window, "_NET_WM_WINDOW_TYPE_DOCK");
+	} else {
+		atoms[0] = ATOMGDK (window, "_NET_WM_WINDOW_TYPE_DOCK");
+	}
 
         XChangeProperty (GDK_WINDOW_XDISPLAY (window),
                          GDK_WINDOW_XWINDOW (window),
-                         XInternAtom (gdk_display,
-                                      "_NET_WM_WINDOW_TYPE",
-                                      False),
+			 ATOMGDK (window, "_NET_WM_WINDOW_TYPE"),
                          XA_ATOM, 32, PropModeReplace,
-                         (guchar *)&atom, 1);
+                         (guchar *)atoms, 
+			 autohide ? 2 : 1);
+}
+
+void
+xstuff_set_wmspec_strut (GdkWindow *window,
+			 int left,
+			 int right,
+			 int top,
+			 int bottom)
+{
+	long vals[4];
+        
+	vals[0] = left;
+	vals[1] = right;
+	vals[2] = top;
+	vals[3] = bottom;
+
+        XChangeProperty (GDK_WINDOW_XDISPLAY (window),
+                         GDK_WINDOW_XWINDOW (window),
+			 ATOMGDK (window, "_NET_WM_STRUT"),
+                         XA_CARDINAL, 32, PropModeReplace,
+                         (guchar *)vals, 4);
+}
+
+void
+xstuff_delete_property (GdkWindow *window, const char *name)
+{
+        XDeleteProperty (GDK_WINDOW_XDISPLAY (window),
+                         GDK_WINDOW_XWINDOW (window),
+			 ATOMGDK (window, name));
 }
